from __future__ import annotations

from tcsoa.gen.ProjectManagement._2012_02.ScheduleManagement import TaskCreateContainer, CreatedDependenciesContainer, AssignmentUpdateContainer, DeleteTaskContainer, DependencyCreateContainer, DependencyUpdateContainer, LaunchedWorkflowContainer, CriticalTasksContainer, MoveRequest, AssignmentCreateContainer, ObjectUpdateContainer, CreatedObjectsContainer
from typing import List
from tcsoa.gen.BusinessObjects import WorkspaceObject, ScheduleTask, Schedule, ResourceAssignment, TaskDependency
from tcsoa.gen.ProjectManagement._2011_06.ScheduleManagement import SchMgtOptions
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService
from datetime import datetime


class ScheduleManagementService(TcService):

    @classmethod
    def launchScheduledWorkflow(cls, tasks: List[ScheduleTask]) -> LaunchedWorkflowContainer:
        """
        A Teamcenter schedule  task can be configured in such a way that when certain conditions are
        met the associated workflow can be initiated. The triggering rules or conditions
        create a workflow process. If there are updates to the workflow process tasks, a
        notification is sent to the Teamcenter schedule task so that the schedule task can be
        updated. This operation launches workflows associated with the tasks .
        
        """
        return cls.execute_soa_method(
            method_name='launchScheduledWorkflow',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'tasks': tasks},
            response_cls=LaunchedWorkflowContainer,
        )

    @classmethod
    def moveTasks(cls, schedule: Schedule, moveRequests: List[MoveRequest]) -> ServiceData:
        """
        Allow schedule tasks and proxy tasks to be moved to a different location in the task hierarchy.
        The schedule task move requests are specified in the 'MoveRequest' structure, which store information about
        schedule task object, its new parent and its new previous sibling. Additional errors during this operation will
        be returned in the list of partial errors in the 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='moveTasks',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'moveRequests': moveRequests},
            response_cls=ServiceData,
        )

    @classmethod
    def recalculateScheduleNonInteractive(cls, schedule: Schedule, recalcType: int, runAsync: bool) -> ServiceData:
        """
        This operation performs the revalidation/rerunning of the business logic on the properties of the schedule and
        its child objects based on the requested properties flag or ALL. Interactive use of this operation is not
        recommended.
        """
        return cls.execute_soa_method(
            method_name='recalculateScheduleNonInteractive',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'recalcType': recalcType, 'runAsync': runAsync},
            response_cls=ServiceData,
        )

    @classmethod
    def scaleScheduleNonInteractive(cls, schedule: Schedule, scaleFactor: float, options: SchMgtOptions) -> ServiceData:
        """
        The scale schedule operation allows for the user to scale the schedule based on specified lag time.
        This operation throws a 'ServiceException' in case of failure. The service exception will contain the error
        message of the failure. Additional errors will be returned in the list of partial errors in the 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='scaleScheduleNonInteractive',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'scaleFactor': scaleFactor, 'options': options},
            response_cls=ServiceData,
        )

    @classmethod
    def shiftScheduleNonInteractive(cls, schedule: Schedule, newDate: datetime, newFinish: bool) -> ServiceData:
        """
        Shifts the schedule forward or backwards to the new provided start/finish date.
        """
        return cls.execute_soa_method(
            method_name='shiftScheduleNonInteractive',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'newDate': newDate, 'newFinish': newFinish},
            response_cls=ServiceData,
        )

    @classmethod
    def updateAssignments(cls, schedule: Schedule, assignmentUpdates: List[AssignmentUpdateContainer]) -> ServiceData:
        """
        The operation updates the resource assignments in a given schedule. Only the resource load of the assignment
        can be updated with this operation. It returns the 'ServiceData' which will have information of updated
        assignment objects. This operation throws a 'ServiceException' in case of failure. The service exception will
        contain the error message of the failure. Additional errors will be returned in the list of partial errors in
        the 'ServiceData'. 
        """
        return cls.execute_soa_method(
            method_name='updateAssignments',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'assignmentUpdates': assignmentUpdates},
            response_cls=ServiceData,
        )

    @classmethod
    def updateDependencies(cls, schedule: Schedule, dependencyUpdates: List[DependencyUpdateContainer]) -> ServiceData:
        """
        This operation  updates the dependencies in a given schedule. It takes an array of 'DependencyUpdateContainer'
        objects in which the type of the dependency,  and lag time updates can be specified per dependency object.
        """
        return cls.execute_soa_method(
            method_name='updateDependencies',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'dependencyUpdates': dependencyUpdates},
            response_cls=ServiceData,
        )

    @classmethod
    def updateSchedules(cls, scheduleUpdates: List[ObjectUpdateContainer]) -> ServiceData:
        """
        Updates all the affected scheduling objects based on the initial users request to the application interface. 
        Properties on the Schedule object like object name, description, start date, finish date, status, is schedule
        template, wbsformat,  customer name, customer number , published, is public and priority etc can be updated
        with  this operation.
        """
        return cls.execute_soa_method(
            method_name='updateSchedules',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'scheduleUpdates': scheduleUpdates},
            response_cls=ServiceData,
        )

    @classmethod
    def updateTasks(cls, schedule: Schedule, updates: List[ObjectUpdateContainer]) -> ServiceData:
        """
        Updates the specified tasks in a given schedule. Task updates are specified in the 'ObjectUpdateContainer' 
        structure. All the scheduling data properties and execution data properties on the schedule task can be
        updated. Updates on the summary tasks are not allowed. This operation supports updating the custom attributes
        of OOTB schedule tasks  as well as custom schedule tasks. The updated objects are returned back in the service
        data of the response. This operation throws a 'ServiceException' in case of failure. The service exception will
        contain the error message of the failure. Additional errors will be returned in the list of partial errors in
        the 'ServiceData'.
        
        
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='updateTasks',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'updates': updates},
            response_cls=ServiceData,
        )

    @classmethod
    def createDependencies(cls, schedule: Schedule, newDependencies: List[DependencyCreateContainer]) -> CreatedDependenciesContainer:
        """
        Creates 'Dependencies' between tasks in the same schedule, between a task and a proxy task in the same
        schedule, or between a tasks in different schedules (but in the same master schedule).  It returns the created
        dependencies, created proxy tasks (if any), and the objects affected by this change. This operation throws a
        'ServiceException' in case of failure. The service exception will contain the error message of the failure.
        Additional errors will be returned in the list of partial errors in the 'ServiceData'. 
        
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='createDependencies',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'newDependencies': newDependencies},
            response_cls=CreatedDependenciesContainer,
        )

    @classmethod
    def createTasks(cls, schedule: Schedule, createContainers: List[TaskCreateContainer]) -> CreatedObjectsContainer:
        """
        Creates the specified tasks in a given schedule. The information needed to create task are specified in the
        'TaskCreateContainer' structure. It returns 'CreatedObjectsContainer' which will have information of created
        tasks, updated tasks and 'ServiceData'. Throws a 'ServiceException' in case of failure. The service exception
        will contain the error message of the failure. Additional errors will be returned in the list of partial errors
        in the 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='createTasks',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'createContainers': createContainers},
            response_cls=CreatedObjectsContainer,
        )

    @classmethod
    def deleteAssignments(cls, schedule: Schedule, assignmentDeletes: List[ResourceAssignment]) -> ServiceData:
        """
        This operation deletes resource assignments from the tasks in the given schedule. It takes schedule from which
        assignments is to be deleted and array of resource assignments to be deleted as input.
        The deleted resource assignment objects are returned in the 'deletedObjects' of the 'ServiceData'. Operation
        throws a 'ServiceException' in case of failure. The service exception will contain the error message of the
        failure. Additional errors will be returned in the list of partial errors in the 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='deleteAssignments',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'assignmentDeletes': assignmentDeletes},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteDependencies(cls, schedule: Schedule, dependencyDeletes: List[TaskDependency]) -> ServiceData:
        """
        This operation deletes the dependencies between the tasks in a given Schedule. 
        """
        return cls.execute_soa_method(
            method_name='deleteDependencies',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'dependencyDeletes': dependencyDeletes},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteTasks(cls, schedule: Schedule, deleteTasks: List[WorkspaceObject]) -> DeleteTaskContainer:
        """
        Deletes the requested tasks along with the sub tasks, assignments, dependencies, and task deliverables.
        """
        return cls.execute_soa_method(
            method_name='deleteTasks',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'deleteTasks': deleteTasks},
            response_cls=DeleteTaskContainer,
        )

    @classmethod
    def findCriticalPathTasks(cls, schedule: Schedule) -> CriticalTasksContainer:
        """
        This operation finds and returns the tasks on the critical path of the schedule. The critical path is the task
        or tasks that would likely affect the last task in the schedule if they were completed late. The critical path
        is calculated by determining the last task in the project (time wise). Any task where a slip would delay the
        last task in the project is on the critical path. Tasks linked by dependencies have a longer critical path
        (chain of tasks). The tasks on the critical path with the longest sequence of dependent tasks merit the most
        attention to on-time completion in order to avoid delays.
        """
        return cls.execute_soa_method(
            method_name='findCriticalPathTasks',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule},
            response_cls=CriticalTasksContainer,
        )

    @classmethod
    def assignResources(cls, schedule: Schedule, createAssignments: List[AssignmentCreateContainer]) -> CreatedObjectsContainer:
        """
        This operation assigns resources (Users and Disciplines) to tasks in a given schedule. The operation returns
        the 'CreatedObjectsContainer' which will have information of created assignments, updated tasks and service
        data.The operation throws a 'ServiceException' in case of failure. The service exception will contain the error
        message of the failure. 
        
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='assignResources',
            library='ProjectManagement',
            service_date='2012_02',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'createAssignments': createAssignments},
            response_cls=CreatedObjectsContainer,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Fnd0Profiler
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class PerformanceInfoResponse(TcBaseObj):
    """
    A structure of vector of 'Fnd0Profiler' and 'ServiceData'.
    
    :var performanceServiceData: Partial Errors if any.
    :var performanceDataVec: List of 'Fnd0Profiler' objects, one for each service operation request executed.
    """
    performanceServiceData: ServiceData = None
    performanceDataVec: List[Fnd0Profiler] = ()



from __future__ import annotations

from tcsoa.gen.Internal.Configurator._2018_11.ConfiguratorManagement import ConfigurationRosterCoverage, ConfigurationRosterInput
from tcsoa.base import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def getConfigurationRosterCoverage(cls, configurationRoster: ConfigurationRosterInput) -> ConfigurationRosterCoverage:
        """
        The Service operation expects a user to provide a set of configurations which are to be checked for validity
        and completeness. The service operation also returns a set of configurations which are unknown to the user.
        
        These configurations are passed to and returned by the service operation via a &lsquo;Roster Table&rsquo;. 
        
        A &lsquo;Roster Table&rsquo; is used to represent configurations. A &lsquo;Roster Table&rsquo; is capable of
        storing configurations. A configuration record in &lsquo;Roster Table&rsquo; is similar to saved variant rule.
        &lsquo;Roster Table&rsquo; consists of two parts,
        
        1.    Column Headers &ndash; A list of config expressions, one for each column in the table. A config
        expression is used to identify family name, family namespace, value and operator. Example: In the config
        Expression "[Teamcenter]A=A1", family name is A, namespace is "Teamcenter", operator is "=" and operand is "A1".
         
        2.    Rows &ndash; Rows store multiple row. A row contains a configuration, just like a Saved Variant Rule. The
        configuration is stored in a bit string, which means it can only contain &lsquo;0&rsquo; or &lsquo;1&rsquo;. 
        The total length of bit string is twice the number of columns present in &lsquo;Roster Table&rsquo;. Hence a
        string representing a cell has a size of 2.  Given string can only have either &lsquo;0&rsquo; or
        &lsquo;1&rsquo; the following strings can represent a cell. They are described below with their logical meaning
            a.    &lsquo;00&rsquo; &ndash; Both True and False conditions are possible for corresponding column header
        config Expression.
            b.    &lsquo;01&rsquo; &ndash; Only True condition is possible for corresponding column header config
        Expression.
            c.    &lsquo;10&rsquo; &ndash; Only False Condition is possible for corresponding column header config
        Expression.
            d.    &lsquo;11&rsquo; -  Not Defined
        
        A user input consisting of one/more configurations is passed to the service operation using &lsquo;Roster
        Table&rsquo;. These configurations are checked by the service operation for validity and completeness. An input
        &lsquo;Roster Table&rsquo; is said to be complete if the input &lsquo;Roster Table&rsquo; passed by user is
        able to represent all the possible combinations given no constraint is violated.
        
        User has ability to provide scope to the service operation while doing completeness check. This is done by
        passing a list of families. Only families present in the list are used by the service operation to compute the
        possible combinations and then evaluate whether the passed configurations via input &lsquo;Roster Table&rsquo;
        are able to cover each and every one of them.
        
        User also has an ability to provide &lsquo;explicit families&rsquo; to service operation. When a configuration
        from input &lsquo;Roster Table&rsquo; is checked for validity turns out to be &lsquo;invalid&rsquo; then
        service operation checks weather the reason for its invalidity is due to a family present in &lsquo;explicit
        families&rsquo;. If yes, then exception conditions on those family is checked, if we find an exception
        condition that meets the configuration input this implies the exception condition allows the configuration to
        be valid.
        
        Given all configurations passed via input &lsquo;Roster Table&rsquo; are valid, and they are able to represent
        all possible combinations then input is said to be complete. If they don&rsquo;t the configurations passed via
        input &lsquo;Roster Table&rsquo; are termed incomplete. Missing configurations are computed by service
        operation and returned via output &lsquo;Roster Table&rsquo;
        
        If some but not all configuration passed by user via &lsquo;Roster Table&rsquo; is found to be invalid, this
        means that the configuration wasn&rsquo;t able to satisfy constraints present in the system. User can specify
        severity of constraints which will participate in evaluating the configurations in input &lsquo;Roster
        Table&rsquo;. These invalid configurations are communicated back to the user and missing configurations are
        evaluated using valid configurations present in input &lsquo;Roster Table&rsquo;.
        
        If all the configurations passed to the service operation are invalid the service operation will return empty
        output &lsquo;Roster Table&rsquo;.
        
        Use Case 1: 
        A product context has 3 Boolean Families &lsquo;A&rsquo;, &lsquo;B&rsquo; and &lsquo;C&rsquo;.
        
        These 3 families can create a total of 8 configurations given there is no constraint in the system.
        
        A user can use &lsquo;getConfigurationRosterCoverage&rsquo; service operation to pass 3 configuration among the
        8 possible via input &lsquo;Roster Table&rsquo;. 
        
        In the current example all the 8 configurations are possible but the user only passed 3 of them in input. Hence
        the service operation provides verdict as &lsquo;valid&rsquo; for all the 3 input configurations. The remaining
        5 configurations are returned back to user via output &lsquo;Roster Table&rsquo;. Additionally service
        operation also returns a verdict of &lsquo;incomplete&rsquo; to user as he missed 5 possible configurations in
        input &lsquo;Roster Table&rsquo;.
        """
        return cls.execute_soa_method(
            method_name='getConfigurationRosterCoverage',
            library='Internal-Configurator',
            service_date='2018_11',
            service_name='ConfiguratorManagement',
            params={'configurationRoster': configurationRoster},
            response_cls=ConfigurationRosterCoverage,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Administration._2019_06.UserManagement import GetGroupRoleViewModelInput, GetGroupRoleViewModelResponse
from tcsoa.base import TcService


class UserManagementService(TcService):

    @classmethod
    def getGroupRoleViewModelRows(cls, input: GetGroupRoleViewModelInput) -> GetGroupRoleViewModelResponse:
        """
        This operation returns the list of view models which can be later consumed in Active Workspace client. The view
        model captures the information about the group and associated default role for the group where the input user
        has active membership. Active Workspace table widgets are capable of consuming these view models and each view
        model will represent a row in the Active Workspace table widget. 
        
        Normally Active Workspace table when rendering business objects shown object represented by a rows and
        specified properties on the object as columns. But in few special cases where we really can&rsquo;t show the
        object properties as is on the column, there we need some way out. This ViewModel approach helps in handling
        such cases. For Example: User&rsquo;s Groups and default Role associated with those groups is actually a
        boolean property with the data model. But on Active Workspace UI we want to show this default role as a string
        LOV. To handle this conversion we make use of ViewModel object.
        """
        return cls.execute_soa_method(
            method_name='getGroupRoleViewModelRows',
            library='Internal-Administration',
            service_date='2019_06',
            service_name='UserManagement',
            params={'input': input},
            response_cls=GetGroupRoleViewModelResponse,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetStepP21FileCountsResponse(TcBaseObj):
    """
    The counts of available STEP P21 product files for the given classes will be returned.
    The following partial errors may be returned:
     - 71513 Invalid class ID
     - 300361 Assortment file cannot be found.
    
    :var serviceData: The service data
    :var countMap: A map of class IDs for which the counts are requested and its corresponding counts.
    """
    serviceData: ServiceData = None
    countMap: MapStringInt = None


@dataclass
class GetVendorCatalogInfoResponse(TcBaseObj):
    """
    A list of all vendor catalogs that are available on the Teamcenter server machine. Each catalog info object
    contains detailed information about each catalog like the name, acronamy, version, language and description.
    Additionally a unique catalog ID is returned that can be used to identify it lateron. The root class ID is the ID
    of the Classification main class for this specific vendor catalog, when it is imported. The root directory that is
    returned is the actual, full directory where the catalog is located. It can be used to start the import of the
    catalog hierarchy using the service operation importVendorCatalogHierarchy.
    Only valid catalogs will be returned. If there is a corrupt catalog (e.g., if one of the mandatory files or
    directories is missing), this catalog will not be returned in the list of catalogs. If one corrupt catalog is
    found, the system continues searching for other valid catalogs and returns all valid ones.
    The following partial errors may be returned:
     - 300300 The preference MRMGTCVendorCatalogRootDir is not defined.
     - 300301 The specified vendor catalog root directory does not exist.
     - 300302 No vendor catalog was found.
    
    
    
    :var serviceData: The service data
    :var catalogInfo: The vendor catalog information
    """
    serviceData: ServiceData = None
    catalogInfo: List[CatalogInfo] = ()


@dataclass
class ImportStepP21FilesResponse(TcBaseObj):
    """
    The importStepP21FilesResponse object that contains the logfile with additional data of the import. When there is a
    failure during the import, an error code will be returned as the part of the service data.
    Possible errors are:
     - 71513 Invalid class ID
     - 300361 Assortment file cannot be found
     - 300362 Mapping file cannot be found
     - 300363 STEP P21 file cannot be found
     - 300324 Invalid import options
     - 300364 Error importing STEP P21 files (details can be found in the import log file (file ticket))
    
    :var serviceData: The service data
    :var logFileTicket: Ticket of the import log file
    """
    serviceData: ServiceData = None
    logFileTicket: RMFileTicket = None


@dataclass
class ImportVendorCatalogHierarchyResponse(TcBaseObj):
    """
    This importVendorCatalogHierarchyResponse object that contains the logfile with additional data for the import.
    When there is a failure during the import, an error code will be returned as the part of the service data.
    
    Possible errors are:
     - 300301 Specified vendor catalog root directory does not exist
     - 300320 Transfer mode cannot be found
     - 300321 Invalid transfer mode tag
     - 300322 Transfer mode object does not exist
     - 300323 Vendor catalog hierarchy import file does not exist
     - 300324 Invalid import options
     - 300325 File ticket for PLMXML log file cannot be created
     - 300326 Error importing vendor catalog hierarchy (details can be found in the import log file (file ticket))
    
    
    
    :var serviceData: Service data that can contain error codes
    :var logFileTicket: Ticket of the import log file
    """
    serviceData: ServiceData = None
    logFileTicket: RMFileTicket = None


@dataclass
class RMFileTicket(TcBaseObj):
    """
    The file information of log file
    
    :var ticket: The FMS file ticket
    :var fileName: The original file name
    """
    ticket: str = ''
    fileName: str = ''


@dataclass
class CatalogInfo(TcBaseObj):
    """
    The structure contains vendor catalog information
    
    :var vendorName: Name of the vendor that provided the catalog
    :var vendorAcronym: Acronym of the vendor that provided the catalog
    :var vendorCatalogVersion: Version of the catalog
    :var vendorCatalogLanguage: Language of the catalog
    :var vendorCatalogDescription: Description of the catalog
    :var vendorCatalogShortDescription: Short description of the catalog
    :var vendorCatalogID: Unique ID of the catalog
    :var vendorCatalogRootClassID: Root class ID of the catalog
    :var vendorCatalogRootDir: Root directory of the catalog
    """
    vendorName: str = ''
    vendorAcronym: str = ''
    vendorCatalogVersion: str = ''
    vendorCatalogLanguage: str = ''
    vendorCatalogDescription: str = ''
    vendorCatalogShortDescription: str = ''
    vendorCatalogID: str = ''
    vendorCatalogRootClassID: str = ''
    vendorCatalogRootDir: str = ''


"""
Map String Int
"""
MapStringInt = Dict[str, int]



from __future__ import annotations

from tcsoa.gen.Vendormanagement._2007_06.VendorManagement import CreateVendorsResponse, CreateVendorPartsResponse, LineItemProps, VendorProperties, BidPackageProps, VendorPartProperties, CreateBidPacksResponse
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class VendorManagementService(TcService):

    @classmethod
    def deleteVendorRoles(cls, input: List[VendorProperties]) -> ServiceData:
        """
        This service operation deletes specified VendorRole objects attached with VendorRevision objects mentioned
        through 'VendorProperties'  data structure. VendorRevision is specified through vendor ID and Revision name in
        'VendorProperties'. The combination to be specified thus is 'vendorId + revId + roleType'. So, this combination
        denotes that there will be one instance of 'VendorProperties' per VendorRole to be deleted. In case of
        VendorRevision is not found based on the Id and Revision key, this operation will return an error.
        """
        return cls.execute_soa_method(
            method_name='deleteVendorRoles',
            library='Vendormanagement',
            service_date='2007_06',
            service_name='VendorManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteVendors(cls, input: List[VendorProperties]) -> ServiceData:
        """
        This service operation deletes Vendor objects and associated VendorRevision objects and VendorRole objects.
        When provided with the input in the form of 'VendorProperties' structure, the operation finds the specified
        Vendor objects and deletes them. VendorRole objects  will also be deleted along with the associated
        VendorRevision objects and Vendor objects.
        """
        return cls.execute_soa_method(
            method_name='deleteVendors',
            library='Vendormanagement',
            service_date='2007_06',
            service_name='VendorManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def createOrUpdateBidPackages(cls, properties: List[BidPackageProps], container: BusinessObject, relationType: str) -> CreateBidPacksResponse:
        """
        This service operation creates or updates a group of BidPackage, BidPackageRevision objects. This operation
        allows the user to find or create a set of BidPackage objects based on the input data. The service first tries
        to find the existence of the specified BidPackage or BidPackageRevision. If the specified BidPackage objects
        exist and any of the input attribute values differ from those already set, the BidPackage objects will be
        updated.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateBidPackages',
            library='Vendormanagement',
            service_date='2007_06',
            service_name='VendorManagement',
            params={'properties': properties, 'container': container, 'relationType': relationType},
            response_cls=CreateBidPacksResponse,
        )

    @classmethod
    def createOrUpdateLineItems(cls, properties: List[LineItemProps], bidPackage: BusinessObject) -> ServiceData:
        """
        This service operation creates or updates a group of BidPackageLineItem objects in the context of the mentioned
        BidPackage. This operation allows the user to find or create a set of BidPackageLineItem objects based on the
        input data. It first tries to find the existence of the specified BidPackageLineItem for the specified
        BidPackage. If the specified BidPackageLineItem is found, then its corresponding data will be updated.
        Otherwise, it will create the specified BidPackageLineItem and associate it to the specified
        BidPackageRevision. If the specified BidPackageLineItem objects exist and any of the input attribute values
        differ from those already set, the BidPackageLineItem objects will be updated.
        
        The operation is now deprecated. It is now replaced by
        'Teamcenter::Soa::Vendormanagement::VendorManagement::createOrUpdateLineItemsWithType.'
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateLineItems',
            library='Vendormanagement',
            service_date='2007_06',
            service_name='VendorManagement',
            params={'properties': properties, 'bidPackage': bidPackage},
            response_cls=ServiceData,
        )

    @classmethod
    def createOrUpdateVendorParts(cls, properties: List[VendorPartProperties], container: BusinessObject, relationType: str) -> CreateVendorPartsResponse:
        """
        This service operation creates or updates a group of VendorPart objects or CommericialPart objects. The choice
        could be either CommercialPart or VendorPart only. This operation allows the user to update or create a set of
        CommercialPart objects or VendorPart objects based on the input data. It first tries to find the existence of
        the specified VendorPart or VendorPartRevision. If the specified parts are found, those parts will be updated
        with the specified values. Otherwise new parts will be created. 
        The choice for type of part is given through data member type of 'VendorPartProperties' structure. Only
        'ManufacturerPart' and 'CommercialPart' are valid types. 
        Behavior for CommercialPart:  The service will create the CommercialPart and CommercialPartRevision and it will
        associate the created CommercialPart to the specified parent (container). It also associates the part to the
        specified Vendor object if given. 
        Behavior for ManufacturerPart:  The service will create the ManufacturerPart and ManufacturerPartRevision and
        will associate the part to the specified Vendor object. It also associates the created ManufacturerPart to the
        CommercialPart if specified.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateVendorParts',
            library='Vendormanagement',
            service_date='2007_06',
            service_name='VendorManagement',
            params={'properties': properties, 'container': container, 'relationType': relationType},
            response_cls=CreateVendorPartsResponse,
        )

    @classmethod
    def createOrUpdateVendors(cls, properties: List[VendorProperties], container: BusinessObject, relationType: str) -> CreateVendorsResponse:
        """
        This service operation creates or updates a group of Vendor, VendorRevision Objects and VendorRole objects. It
        allows the user to update or create a set of Vendor objects based on the input data. The service first tries to
        find the existence of the specified Vendor or VendorRevision or VendorRole. If the service is able to find any
        of those objects, then those objects will be updated. If the service is not able to find those objects, then
        those objects will be created. If the Vendor exists, but VendorRevision does not, then VendorRevision and
        VendorRole (and VendorRole form) will be created. If the specified Vendor object and its associated objects
        exist and any of the input attribute values differ from those already set, the Vendor and its associated
        objects will be updated.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateVendors',
            library='Vendormanagement',
            service_date='2007_06',
            service_name='VendorManagement',
            params={'properties': properties, 'container': container, 'relationType': relationType},
            response_cls=CreateVendorsResponse,
        )

    @classmethod
    def deleteLineItems(cls, input: List[LineItemProps], bidPackage: BidPackageProps) -> ServiceData:
        """
        This service operation deletes the BidPackageLineItem objects associated with a specific BidPackageRevision.
        The BidPackageLineItem objects to be deleted are searched with their names provided through the input data
        structures 'LineItemProps'. Hence there will be one entry in the input vector 'input' per BidPackageLineItem to
        be deleted. There will be one call each for every BidPackageRevision for which BidPackageLineItem objects are
        to be deleted.
        """
        return cls.execute_soa_method(
            method_name='deleteLineItems',
            library='Vendormanagement',
            service_date='2007_06',
            service_name='VendorManagement',
            params={'input': input, 'bidPackage': bidPackage},
            response_cls=ServiceData,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tcsoa',
 'tcsoa.fcc',
 'tcsoa.gen',
 'tcsoa.gen.AWS2',
 'tcsoa.gen.AWS2._2016_12',
 'tcsoa.gen.AWS2._2017_06',
 'tcsoa.gen.AWS2._2018_12',
 'tcsoa.gen.ActiveCollaboration',
 'tcsoa.gen.ActiveCollaboration._2020_12',
 'tcsoa.gen.ActiveWorkspaceBom',
 'tcsoa.gen.ActiveWorkspaceBom._2015_10',
 'tcsoa.gen.ActiveWorkspaceVis',
 'tcsoa.gen.ActiveWorkspaceVis._2015_03',
 'tcsoa.gen.Administration',
 'tcsoa.gen.Administration._2006_03',
 'tcsoa.gen.Administration._2007_01',
 'tcsoa.gen.Administration._2007_06',
 'tcsoa.gen.Administration._2008_03',
 'tcsoa.gen.Administration._2008_06',
 'tcsoa.gen.Administration._2008_12',
 'tcsoa.gen.Administration._2010_04',
 'tcsoa.gen.Administration._2011_05',
 'tcsoa.gen.Administration._2012_09',
 'tcsoa.gen.Administration._2012_10',
 'tcsoa.gen.Administration._2014_10',
 'tcsoa.gen.Administration._2015_03',
 'tcsoa.gen.Administration._2015_07',
 'tcsoa.gen.Administration._2016_03',
 'tcsoa.gen.Administration._2016_10',
 'tcsoa.gen.Administration._2017_05',
 'tcsoa.gen.Administration._2018_11',
 'tcsoa.gen.Ai',
 'tcsoa.gen.Ai._2006_03',
 'tcsoa.gen.Ai._2007_12',
 'tcsoa.gen.Ai._2008_05',
 'tcsoa.gen.Ai._2008_06',
 'tcsoa.gen.Ai._2009_06',
 'tcsoa.gen.Ai._2009_10',
 'tcsoa.gen.Ai._2010_09',
 'tcsoa.gen.Ai._2012_09',
 'tcsoa.gen.Ai._2013_05',
 'tcsoa.gen.Ai._2013_12',
 'tcsoa.gen.Ai._2014_12',
 'tcsoa.gen.Ai._2018_06',
 'tcsoa.gen.Allocations',
 'tcsoa.gen.Allocations._2007_01',
 'tcsoa.gen.Allocations._2011_06',
 'tcsoa.gen.AuthorizedDataAccess',
 'tcsoa.gen.AuthorizedDataAccess._2007_06',
 'tcsoa.gen.AuthorizedDataAccess._2009_10',
 'tcsoa.gen.AuthorizedDataAccess._2012_09',
 'tcsoa.gen.AuthorizedDataAccess._2013_05',
 'tcsoa.gen.AuthorizedDataAccess._2017_05',
 'tcsoa.gen.AuthorizedDataAccess._2018_06',
 'tcsoa.gen.Bom',
 'tcsoa.gen.Bom._2008_06',
 'tcsoa.gen.Bom._2010_09',
 'tcsoa.gen.BusinessModeler',
 'tcsoa.gen.BusinessModeler._2007_06',
 'tcsoa.gen.BusinessModeler._2008_06',
 'tcsoa.gen.BusinessModeler._2010_04',
 'tcsoa.gen.BusinessModeler._2011_06',
 'tcsoa.gen.Cad',
 'tcsoa.gen.Cad._2007_01',
 'tcsoa.gen.Cad._2007_06',
 'tcsoa.gen.Cad._2007_09',
 'tcsoa.gen.Cad._2007_12',
 'tcsoa.gen.Cad._2008_03',
 'tcsoa.gen.Cad._2008_06',
 'tcsoa.gen.Cad._2009_04',
 'tcsoa.gen.Cad._2010_09',
 'tcsoa.gen.Cad._2011_06',
 'tcsoa.gen.Cad._2012_09',
 'tcsoa.gen.Cad._2013_05',
 'tcsoa.gen.Cad._2014_10',
 'tcsoa.gen.Cad._2016_03',
 'tcsoa.gen.Cad._2016_09',
 'tcsoa.gen.Cad._2018_06',
 'tcsoa.gen.Cad._2019_06',
 'tcsoa.gen.Cad._2020_01',
 'tcsoa.gen.Cae',
 'tcsoa.gen.Cae._2009_10',
 'tcsoa.gen.Cae._2011_06',
 'tcsoa.gen.Cae._2012_02',
 'tcsoa.gen.Cae._2013_12',
 'tcsoa.gen.Cae._2014_06',
 'tcsoa.gen.CalendarManagement',
 'tcsoa.gen.CalendarManagement._2007_01',
 'tcsoa.gen.CalendarManagement._2007_06',
 'tcsoa.gen.CalendarManagement._2008_06',
 'tcsoa.gen.ChangeManagement',
 'tcsoa.gen.ChangeManagement._2008_06',
 'tcsoa.gen.ChangeManagement._2009_06',
 'tcsoa.gen.ChangeManagement._2015_10',
 'tcsoa.gen.ChangeManagement._2020_01',
 'tcsoa.gen.Classification',
 'tcsoa.gen.Classification._2007_01',
 'tcsoa.gen.Classification._2009_10',
 'tcsoa.gen.Classification._2011_06',
 'tcsoa.gen.Classification._2011_12',
 'tcsoa.gen.Classification._2015_03',
 'tcsoa.gen.Classification._2015_10',
 'tcsoa.gen.Classification._2016_03',
 'tcsoa.gen.Classification._2016_09',
 'tcsoa.gen.ClassificationCommon',
 'tcsoa.gen.ClassificationCommon._2020_01',
 'tcsoa.gen.ClassificationCommon._2020_12',
 'tcsoa.gen.ConfigFilterCriteria',
 'tcsoa.gen.ConfigFilterCriteria._2011_06',
 'tcsoa.gen.ConfigFilterCriteria._2013_05',
 'tcsoa.gen.ConfigFilterCriteria._2017_05',
 'tcsoa.gen.Configuration',
 'tcsoa.gen.Configuration._2010_04',
 'tcsoa.gen.Configurator',
 'tcsoa.gen.Configurator._2014_12',
 'tcsoa.gen.Core',
 'tcsoa.gen.Core._2006_03',
 'tcsoa.gen.Core._2007_01',
 'tcsoa.gen.Core._2007_06',
 'tcsoa.gen.Core._2007_09',
 'tcsoa.gen.Core._2007_12',
 'tcsoa.gen.Core._2008_03',
 'tcsoa.gen.Core._2008_05',
 'tcsoa.gen.Core._2008_06',
 'tcsoa.gen.Core._2009_04',
 'tcsoa.gen.Core._2009_10',
 'tcsoa.gen.Core._2010_04',
 'tcsoa.gen.Core._2010_09',
 'tcsoa.gen.Core._2011_06',
 'tcsoa.gen.Core._2012_02',
 'tcsoa.gen.Core._2012_09',
 'tcsoa.gen.Core._2012_10',
 'tcsoa.gen.Core._2013_05',
 'tcsoa.gen.Core._2014_06',
 'tcsoa.gen.Core._2014_10',
 'tcsoa.gen.Core._2015_07',
 'tcsoa.gen.Core._2015_10',
 'tcsoa.gen.Core._2016_05',
 'tcsoa.gen.Core._2016_09',
 'tcsoa.gen.Core._2016_10',
 'tcsoa.gen.Core._2017_05',
 'tcsoa.gen.Core._2017_11',
 'tcsoa.gen.Core._2018_06',
 'tcsoa.gen.Core._2018_11',
 'tcsoa.gen.Core._2019_06',
 'tcsoa.gen.Core._2020_01',
 'tcsoa.gen.Core._2020_04',
 'tcsoa.gen.Diagramming',
 'tcsoa.gen.Diagramming._2011_06',
 'tcsoa.gen.Diagramming._2012_09',
 'tcsoa.gen.Diagramming._2014_06',
 'tcsoa.gen.DocMgmtAw',
 'tcsoa.gen.DocMgmtAw._2017_06',
 'tcsoa.gen.DocumentManagement',
 'tcsoa.gen.DocumentManagement._2007_01',
 'tcsoa.gen.DocumentManagement._2008_06',
 'tcsoa.gen.DocumentManagement._2010_04',
 'tcsoa.gen.DocumentManagement._2011_06',
 'tcsoa.gen.DocumentManagement._2013_12',
 'tcsoa.gen.DocumentManagement._2018_06',
 'tcsoa.gen.DocumentManagement._2018_11',
 'tcsoa.gen.EditContext',
 'tcsoa.gen.EditContext._2014_12',
 'tcsoa.gen.EditContext._2015_07',
 'tcsoa.gen.EditContext._2016_04',
 'tcsoa.gen.EditContext._2016_10',
 'tcsoa.gen.GlobalMultiSite',
 'tcsoa.gen.GlobalMultiSite._2007_06',
 'tcsoa.gen.GlobalMultiSite._2007_12',
 'tcsoa.gen.GlobalMultiSite._2008_06',
 'tcsoa.gen.GlobalMultiSite._2010_04',
 'tcsoa.gen.GlobalMultiSite._2011_06',
 'tcsoa.gen.GlobalMultiSite._2020_04',
 'tcsoa.gen.ImportExport',
 'tcsoa.gen.ImportExport._2007_06',
 'tcsoa.gen.ImportExport._2008_06',
 'tcsoa.gen.ImportExport._2011_06',
 'tcsoa.gen.ImportExport._2012_09',
 'tcsoa.gen.ImportExport._2017_11',
 'tcsoa.gen.Internal',
 'tcsoa.gen.Internal.AWS2',
 'tcsoa.gen.Internal.AWS2._2012_10',
 'tcsoa.gen.Internal.AWS2._2013_12',
 'tcsoa.gen.Internal.AWS2._2014_11',
 'tcsoa.gen.Internal.AWS2._2015_03',
 'tcsoa.gen.Internal.AWS2._2015_10',
 'tcsoa.gen.Internal.AWS2._2016_03',
 'tcsoa.gen.Internal.AWS2._2016_04',
 'tcsoa.gen.Internal.AWS2._2016_12',
 'tcsoa.gen.Internal.AWS2._2017_06',
 'tcsoa.gen.Internal.AWS2._2017_12',
 'tcsoa.gen.Internal.AWS2._2018_05',
 'tcsoa.gen.Internal.AWS2._2018_12',
 'tcsoa.gen.Internal.AWS2._2019_06',
 'tcsoa.gen.Internal.AWS2._2019_12',
 'tcsoa.gen.Internal.AWS2._2020_05',
 'tcsoa.gen.Internal.AWS2._2020_12',
 'tcsoa.gen.Internal.ActiveWorkspaceBom',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2012_10',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2015_03',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2015_07',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2015_10',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2016_03',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2017_06',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2017_12',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2018_05',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2018_12',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2019_06',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2019_12',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2020_05',
 'tcsoa.gen.Internal.ActiveWorkspaceBom._2020_12',
 'tcsoa.gen.Internal.ActiveWorkspaceVis',
 'tcsoa.gen.Internal.ActiveWorkspaceVis._2014_11',
 'tcsoa.gen.Internal.ActiveWorkspaceVis._2015_03',
 'tcsoa.gen.Internal.ActiveWorkspaceVis._2018_05',
 'tcsoa.gen.Internal.Administration',
 'tcsoa.gen.Internal.Administration._2007_06',
 'tcsoa.gen.Internal.Administration._2008_06',
 'tcsoa.gen.Internal.Administration._2009_10',
 'tcsoa.gen.Internal.Administration._2011_06',
 'tcsoa.gen.Internal.Administration._2012_10',
 'tcsoa.gen.Internal.Administration._2013_05',
 'tcsoa.gen.Internal.Administration._2014_10',
 'tcsoa.gen.Internal.Administration._2016_09',
 'tcsoa.gen.Internal.Administration._2017_11',
 'tcsoa.gen.Internal.Administration._2018_11',
 'tcsoa.gen.Internal.Administration._2019_06',
 'tcsoa.gen.Internal.Ai',
 'tcsoa.gen.Internal.Ai._2008_06',
 'tcsoa.gen.Internal.Ai._2016_03',
 'tcsoa.gen.Internal.Ai._2016_04',
 'tcsoa.gen.Internal.AuditManager',
 'tcsoa.gen.Internal.AuditManager._2012_09',
 'tcsoa.gen.Internal.BusinessModeler',
 'tcsoa.gen.Internal.BusinessModeler._2007_01',
 'tcsoa.gen.Internal.BusinessModeler._2010_04',
 'tcsoa.gen.Internal.BusinessModeler._2010_09',
 'tcsoa.gen.Internal.BusinessModeler._2011_06',
 'tcsoa.gen.Internal.BusinessModeler._2013_05',
 'tcsoa.gen.Internal.Cad',
 'tcsoa.gen.Internal.Cad._2008_03',
 'tcsoa.gen.Internal.Cad._2008_05',
 'tcsoa.gen.Internal.Cad._2008_06',
 'tcsoa.gen.Internal.Cad._2010_04',
 'tcsoa.gen.Internal.Cad._2013_05',
 'tcsoa.gen.Internal.Cad._2017_05',
 'tcsoa.gen.Internal.Cae',
 'tcsoa.gen.Internal.Cae._2011_06',
 'tcsoa.gen.Internal.Cae._2013_05',
 'tcsoa.gen.Internal.Cae._2014_06',
 'tcsoa.gen.Internal.ChangeManagement',
 'tcsoa.gen.Internal.ChangeManagement._2012_10',
 'tcsoa.gen.Internal.ChangeManagement._2015_03',
 'tcsoa.gen.Internal.ChangeManagement._2020_01',
 'tcsoa.gen.Internal.Classification',
 'tcsoa.gen.Internal.Classification._2009_10',
 'tcsoa.gen.Internal.Classification._2017_05',
 'tcsoa.gen.Internal.Classification._2018_11',
 'tcsoa.gen.Internal.Classification._2020_04',
 'tcsoa.gen.Internal.ConfigFilterCriteria',
 'tcsoa.gen.Internal.ConfigFilterCriteria._2013_12',
 'tcsoa.gen.Internal.Configurator',
 'tcsoa.gen.Internal.Configurator._2014_06',
 'tcsoa.gen.Internal.Configurator._2014_12',
 'tcsoa.gen.Internal.Configurator._2015_03',
 'tcsoa.gen.Internal.Configurator._2015_10',
 'tcsoa.gen.Internal.Configurator._2016_09',
 'tcsoa.gen.Internal.Configurator._2017_11',
 'tcsoa.gen.Internal.Configurator._2018_06',
 'tcsoa.gen.Internal.Configurator._2018_11',
 'tcsoa.gen.Internal.Core',
 'tcsoa.gen.Internal.Core._2007_01',
 'tcsoa.gen.Internal.Core._2007_06',
 'tcsoa.gen.Internal.Core._2007_09',
 'tcsoa.gen.Internal.Core._2008_06',
 'tcsoa.gen.Internal.Core._2009_10',
 'tcsoa.gen.Internal.Core._2010_04',
 'tcsoa.gen.Internal.Core._2010_09',
 'tcsoa.gen.Internal.Core._2011_06',
 'tcsoa.gen.Internal.Core._2012_02',
 'tcsoa.gen.Internal.Core._2012_09',
 'tcsoa.gen.Internal.Core._2012_10',
 'tcsoa.gen.Internal.Core._2013_05',
 'tcsoa.gen.Internal.Core._2014_10',
 'tcsoa.gen.Internal.Core._2016_10',
 'tcsoa.gen.Internal.Core._2017_05',
 'tcsoa.gen.Internal.Core._2017_11',
 'tcsoa.gen.Internal.Core._2018_06',
 'tcsoa.gen.Internal.Core._2018_11',
 'tcsoa.gen.Internal.Core._2018_12',
 'tcsoa.gen.Internal.Core._2020_01',
 'tcsoa.gen.Internal.Core._2020_04',
 'tcsoa.gen.Internal.DebugMonitor',
 'tcsoa.gen.Internal.DebugMonitor._2011_06',
 'tcsoa.gen.Internal.DebugMonitor._2014_06',
 'tcsoa.gen.Internal.DebugMonitor._2015_07',
 'tcsoa.gen.Internal.DebugMonitor._2015_10',
 'tcsoa.gen.Internal.DebugMonitor._2019_06',
 'tcsoa.gen.Internal.DocMgmtAw',
 'tcsoa.gen.Internal.DocMgmtAw._2019_06',
 'tcsoa.gen.Internal.DocMgmtAw._2019_12',
 'tcsoa.gen.Internal.DocumentManagement',
 'tcsoa.gen.Internal.DocumentManagement._2008_06',
 'tcsoa.gen.Internal.DocumentManagement._2013_05',
 'tcsoa.gen.Internal.DocumentManagement._2013_12',
 'tcsoa.gen.Internal.DocumentManagement._2020_12',
 'tcsoa.gen.Internal.EntCba',
 'tcsoa.gen.Internal.EntCba._2019_12',
 'tcsoa.gen.Internal.Gdis',
 'tcsoa.gen.Internal.Gdis._2006_03',
 'tcsoa.gen.Internal.GlobalMultiSite',
 'tcsoa.gen.Internal.GlobalMultiSite._2007_06',
 'tcsoa.gen.Internal.GlobalMultiSite._2008_06',
 'tcsoa.gen.Internal.GlobalMultiSite._2010_02',
 'tcsoa.gen.Internal.GlobalMultiSite._2017_05',
 'tcsoa.gen.Internal.GlobalMultiSite._2018_11',
 'tcsoa.gen.Internal.GlobalMultiSite._2020_01',
 'tcsoa.gen.Internal.IcsAw',
 'tcsoa.gen.Internal.IcsAw._2017_06',
 'tcsoa.gen.Internal.IcsAw._2017_12',
 'tcsoa.gen.Internal.IcsAw._2018_05',
 'tcsoa.gen.Internal.IcsAw._2018_12',
 'tcsoa.gen.Internal.IcsAw._2019_06',
 'tcsoa.gen.Internal.IcsAw._2019_12',
 'tcsoa.gen.Internal.ImportExport',
 'tcsoa.gen.Internal.ImportExport._2010_04',
 'tcsoa.gen.Internal.ImportExport._2017_11',
 'tcsoa.gen.Internal.ImportExport._2018_06',
 'tcsoa.gen.Internal.Integration',
 'tcsoa.gen.Internal.Integration._2007_06',
 'tcsoa.gen.Internal.Integration._2008_06',
 'tcsoa.gen.Internal.Manufacturing',
 'tcsoa.gen.Internal.Manufacturing._2008_12',
 'tcsoa.gen.Internal.Manufacturing._2011_12',
 'tcsoa.gen.Internal.Manufacturing._2012_09',
 'tcsoa.gen.Internal.Manufacturing._2014_06',
 'tcsoa.gen.Internal.Manufacturing._2014_12',
 'tcsoa.gen.Internal.Manufacturing._2015_03',
 'tcsoa.gen.Internal.Manufacturing._2015_10',
 'tcsoa.gen.Internal.Manufacturing._2016_09',
 'tcsoa.gen.Internal.Manufacturing._2017_05',
 'tcsoa.gen.Internal.Manufacturing._2017_11',
 'tcsoa.gen.Internal.Manufacturing._2018_11',
 'tcsoa.gen.Internal.Manufacturing._2019_06',
 'tcsoa.gen.Internal.Manufacturing._2020_01',
 'tcsoa.gen.Internal.Manufacturing._2020_04',
 'tcsoa.gen.Internal.Mmv',
 'tcsoa.gen.Internal.Mmv._2012_09',
 'tcsoa.gen.Internal.MultiSite',
 'tcsoa.gen.Internal.MultiSite._2007_06',
 'tcsoa.gen.Internal.MultiSite._2011_06',
 'tcsoa.gen.Internal.MultiSite._2012_02',
 'tcsoa.gen.Internal.MultiSite._2017_11',
 'tcsoa.gen.Internal.MultiSite._2018_06',
 'tcsoa.gen.Internal.MultiSite._2020_12',
 'tcsoa.gen.Internal.Notification',
 'tcsoa.gen.Internal.Notification._2014_10',
 'tcsoa.gen.Internal.Notification._2015_03',
 'tcsoa.gen.Internal.Notification._2015_10',
 'tcsoa.gen.Internal.Notification._2017_11',
 'tcsoa.gen.Internal.OccMgmt',
 'tcsoa.gen.Internal.OccMgmt._2020_05',
 'tcsoa.gen.Internal.OccMgmt._2020_12',
 'tcsoa.gen.Internal.ProductConfiguratorAw',
 'tcsoa.gen.Internal.ProductConfiguratorAw._2017_12',
 'tcsoa.gen.Internal.ProductConfiguratorAw._2018_05',
 'tcsoa.gen.Internal.ProductConfiguratorAw._2020_05',
 'tcsoa.gen.Internal.ProductConfiguratorAw._2020_12',
 'tcsoa.gen.Internal.ProjectManagement',
 'tcsoa.gen.Internal.ProjectManagement._2007_01',
 'tcsoa.gen.Internal.ProjectManagement._2007_06',
 'tcsoa.gen.Internal.ProjectManagement._2008_06',
 'tcsoa.gen.Internal.ProjectManagement._2009_10',
 'tcsoa.gen.Internal.ProjectManagement._2010_04',
 'tcsoa.gen.Internal.ProjectManagement._2011_06',
 'tcsoa.gen.Internal.ProjectManagement._2012_02',
 'tcsoa.gen.Internal.ProjectManagement._2014_10',
 'tcsoa.gen.Internal.ProjectManagementAw',
 'tcsoa.gen.Internal.ProjectManagementAw._2019_12',
 'tcsoa.gen.Internal.Query',
 'tcsoa.gen.Internal.Query._2008_06',
 'tcsoa.gen.Internal.Query._2012_02',
 'tcsoa.gen.Internal.Query._2013_05',
 'tcsoa.gen.Internal.Query._2014_10',
 'tcsoa.gen.Internal.Rdv',
 'tcsoa.gen.Internal.Rdv._2007_06',
 'tcsoa.gen.Internal.Rdv._2007_09',
 'tcsoa.gen.Internal.Rdv._2008_03',
 'tcsoa.gen.Internal.Rdv._2009_01',
 'tcsoa.gen.Internal.Rdv._2009_04',
 'tcsoa.gen.Internal.Rdv._2009_05',
 'tcsoa.gen.Internal.Rdv._2010_04',
 'tcsoa.gen.Internal.Reports',
 'tcsoa.gen.Internal.Reports._2007_06',
 'tcsoa.gen.Internal.Requirementsmanagement',
 'tcsoa.gen.Internal.Requirementsmanagement._2009_10',
 'tcsoa.gen.Internal.Requirementsmanagement._2012_02',
 'tcsoa.gen.Internal.Requirementsmanagement._2012_10',
 'tcsoa.gen.Internal.RevRuleMgmt',
 'tcsoa.gen.Internal.RevRuleMgmt._2019_12',
 'tcsoa.gen.Internal.Search',
 'tcsoa.gen.Internal.Search._2020_12',
 'tcsoa.gen.Internal.Security',
 'tcsoa.gen.Internal.Security._2017_12',
 'tcsoa.gen.Internal.Structure',
 'tcsoa.gen.Internal.Structure._2020_12',
 'tcsoa.gen.Internal.StructureManagement',
 'tcsoa.gen.Internal.StructureManagement._2007_06',
 'tcsoa.gen.Internal.StructureManagement._2007_12',
 'tcsoa.gen.Internal.StructureManagement._2008_03',
 'tcsoa.gen.Internal.StructureManagement._2008_05',
 'tcsoa.gen.Internal.StructureManagement._2008_06',
 'tcsoa.gen.Internal.StructureManagement._2009_10',
 'tcsoa.gen.Internal.StructureManagement._2010_04',
 'tcsoa.gen.Internal.StructureManagement._2010_09',
 'tcsoa.gen.Internal.StructureManagement._2011_06',
 'tcsoa.gen.Internal.StructureManagement._2012_02',
 'tcsoa.gen.Internal.StructureManagement._2012_09',
 'tcsoa.gen.Internal.StructureManagement._2013_05',
 'tcsoa.gen.Internal.StructureManagement._2013_12',
 'tcsoa.gen.Internal.StructureManagement._2014_12',
 'tcsoa.gen.Internal.StructureManagement._2015_10',
 'tcsoa.gen.Internal.StructureManagement._2016_03',
 'tcsoa.gen.Internal.StructureManagement._2016_10',
 'tcsoa.gen.Internal.StructureManagement._2017_05',
 'tcsoa.gen.Internal.StructureManagement._2018_11',
 'tcsoa.gen.Internal.StructureManagement._2020_05',
 'tcsoa.gen.Internal.TCXMLImportExport',
 'tcsoa.gen.Internal.TCXMLImportExport._2020_12',
 'tcsoa.gen.Internal.Translation',
 'tcsoa.gen.Internal.Translation._2007_06',
 'tcsoa.gen.Internal.UiConfig',
 'tcsoa.gen.Internal.UiConfig._2014_11',
 'tcsoa.gen.Internal.Validation',
 'tcsoa.gen.Internal.Validation._2007_06',
 'tcsoa.gen.Internal.Validation._2008_06',
 'tcsoa.gen.Internal.Validation._2012_02',
 'tcsoa.gen.Internal.VendorManagementAW',
 'tcsoa.gen.Internal.VendorManagementAW._2019_12',
 'tcsoa.gen.Internal.VendorManagementAW._2020_05',
 'tcsoa.gen.Internal.VendorManagementAW._2020_12',
 'tcsoa.gen.Internal.Vendormanagement',
 'tcsoa.gen.Internal.Vendormanagement._2008_05',
 'tcsoa.gen.Internal.Vendormanagement._2008_06',
 'tcsoa.gen.Internal.Visualization',
 'tcsoa.gen.Internal.Visualization._2008_06',
 'tcsoa.gen.Internal.Visualization._2010_04',
 'tcsoa.gen.Internal.Visualization._2010_09',
 'tcsoa.gen.Internal.Visualization._2011_12',
 'tcsoa.gen.Internal.Visualization._2012_10',
 'tcsoa.gen.Internal.Visualization._2017_05',
 'tcsoa.gen.Internal.Visualization._2018_11',
 'tcsoa.gen.Internal.Workflow',
 'tcsoa.gen.Internal.Workflow._2013_05',
 'tcsoa.gen.Internal.Workflow._2017_11',
 'tcsoa.gen.Internal.Workflowaw',
 'tcsoa.gen.Internal.Workflowaw._2020_12',
 'tcsoa.gen.Internal.XlsBom',
 'tcsoa.gen.Internal.XlsBom._2020_12',
 'tcsoa.gen.Internal_AWS2',
 'tcsoa.gen.Internal_AWS2._2012_10',
 'tcsoa.gen.Internal_AWS2._2013_12',
 'tcsoa.gen.Internal_AWS2._2014_11',
 'tcsoa.gen.Internal_AWS2._2015_03',
 'tcsoa.gen.Internal_AWS2._2015_10',
 'tcsoa.gen.Internal_AWS2._2016_03',
 'tcsoa.gen.Internal_AWS2._2016_04',
 'tcsoa.gen.Internal_AWS2._2016_12',
 'tcsoa.gen.Internal_AWS2._2017_06',
 'tcsoa.gen.Internal_AWS2._2017_12',
 'tcsoa.gen.Internal_AWS2._2018_05',
 'tcsoa.gen.Internal_AWS2._2018_12',
 'tcsoa.gen.Internal_AWS2._2019_06',
 'tcsoa.gen.Internal_AWS2._2019_12',
 'tcsoa.gen.Internal_AWS2._2020_05',
 'tcsoa.gen.Internal_AWS2._2020_12',
 'tcsoa.gen.Internal_ActiveWorkspaceBom',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2012_10',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2015_03',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2015_07',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2015_10',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2016_03',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2017_06',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2017_12',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2018_05',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2018_12',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2019_06',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2019_12',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2020_05',
 'tcsoa.gen.Internal_ActiveWorkspaceBom._2020_12',
 'tcsoa.gen.Internal_ActiveWorkspaceVis',
 'tcsoa.gen.Internal_ActiveWorkspaceVis._2014_11',
 'tcsoa.gen.Internal_ActiveWorkspaceVis._2015_03',
 'tcsoa.gen.Internal_ActiveWorkspaceVis._2018_05',
 'tcsoa.gen.Internal_Administration',
 'tcsoa.gen.Internal_Administration._2007_06',
 'tcsoa.gen.Internal_Administration._2008_06',
 'tcsoa.gen.Internal_Administration._2009_10',
 'tcsoa.gen.Internal_Administration._2011_06',
 'tcsoa.gen.Internal_Administration._2012_10',
 'tcsoa.gen.Internal_Administration._2013_05',
 'tcsoa.gen.Internal_Administration._2014_10',
 'tcsoa.gen.Internal_Administration._2015_10',
 'tcsoa.gen.Internal_Administration._2016_09',
 'tcsoa.gen.Internal_Administration._2017_11',
 'tcsoa.gen.Internal_Administration._2018_06',
 'tcsoa.gen.Internal_Administration._2018_11',
 'tcsoa.gen.Internal_Administration._2019_06',
 'tcsoa.gen.Internal_Ai',
 'tcsoa.gen.Internal_Ai._2008_06',
 'tcsoa.gen.Internal_Ai._2016_03',
 'tcsoa.gen.Internal_Ai._2016_04',
 'tcsoa.gen.Internal_AuditManager',
 'tcsoa.gen.Internal_AuditManager._2012_09',
 'tcsoa.gen.Internal_BusinessModeler',
 'tcsoa.gen.Internal_BusinessModeler._2007_01',
 'tcsoa.gen.Internal_BusinessModeler._2010_04',
 'tcsoa.gen.Internal_BusinessModeler._2010_09',
 'tcsoa.gen.Internal_BusinessModeler._2011_06',
 'tcsoa.gen.Internal_BusinessModeler._2013_05',
 'tcsoa.gen.Internal_Cad',
 'tcsoa.gen.Internal_Cad._2007_12',
 'tcsoa.gen.Internal_Cad._2008_03',
 'tcsoa.gen.Internal_Cad._2008_05',
 'tcsoa.gen.Internal_Cad._2008_06',
 'tcsoa.gen.Internal_Cad._2010_04',
 'tcsoa.gen.Internal_Cad._2013_05',
 'tcsoa.gen.Internal_Cad._2017_05',
 'tcsoa.gen.Internal_Cae',
 'tcsoa.gen.Internal_Cae._2009_10',
 'tcsoa.gen.Internal_Cae._2011_06',
 'tcsoa.gen.Internal_Cae._2012_02',
 'tcsoa.gen.Internal_Cae._2012_09',
 'tcsoa.gen.Internal_Cae._2013_05',
 'tcsoa.gen.Internal_Cae._2013_12',
 'tcsoa.gen.Internal_Cae._2014_06',
 'tcsoa.gen.Internal_ChangeManagement',
 'tcsoa.gen.Internal_ChangeManagement._2012_10',
 'tcsoa.gen.Internal_ChangeManagement._2015_03',
 'tcsoa.gen.Internal_ChangeManagement._2020_01',
 'tcsoa.gen.Internal_Classification',
 'tcsoa.gen.Internal_Classification._2009_10',
 'tcsoa.gen.Internal_Classification._2017_05',
 'tcsoa.gen.Internal_Classification._2018_11',
 'tcsoa.gen.Internal_Classification._2020_04',
 'tcsoa.gen.Internal_CodeCoverage',
 'tcsoa.gen.Internal_CodeCoverage._2011_06',
 'tcsoa.gen.Internal_ConfigFilterCriteria',
 'tcsoa.gen.Internal_ConfigFilterCriteria._2013_12',
 'tcsoa.gen.Internal_Configurator',
 'tcsoa.gen.Internal_Configurator._2014_06',
 'tcsoa.gen.Internal_Configurator._2014_12',
 'tcsoa.gen.Internal_Configurator._2015_03',
 'tcsoa.gen.Internal_Configurator._2015_10',
 'tcsoa.gen.Internal_Configurator._2016_09',
 'tcsoa.gen.Internal_Configurator._2017_11',
 'tcsoa.gen.Internal_Configurator._2018_06',
 'tcsoa.gen.Internal_Configurator._2018_11',
 'tcsoa.gen.Internal_Core',
 'tcsoa.gen.Internal_Core._2006_03',
 'tcsoa.gen.Internal_Core._2007_01',
 'tcsoa.gen.Internal_Core._2007_05',
 'tcsoa.gen.Internal_Core._2007_06',
 'tcsoa.gen.Internal_Core._2007_09',
 'tcsoa.gen.Internal_Core._2007_12',
 'tcsoa.gen.Internal_Core._2008_03',
 'tcsoa.gen.Internal_Core._2008_06',
 'tcsoa.gen.Internal_Core._2009_10',
 'tcsoa.gen.Internal_Core._2010_04',
 'tcsoa.gen.Internal_Core._2010_09',
 'tcsoa.gen.Internal_Core._2011_06',
 'tcsoa.gen.Internal_Core._2012_02',
 'tcsoa.gen.Internal_Core._2012_09',
 'tcsoa.gen.Internal_Core._2012_10',
 'tcsoa.gen.Internal_Core._2013_05',
 'tcsoa.gen.Internal_Core._2014_10',
 'tcsoa.gen.Internal_Core._2014_11',
 'tcsoa.gen.Internal_Core._2016_10',
 'tcsoa.gen.Internal_Core._2017_05',
 'tcsoa.gen.Internal_Core._2017_11',
 'tcsoa.gen.Internal_Core._2018_06',
 'tcsoa.gen.Internal_Core._2018_11',
 'tcsoa.gen.Internal_Core._2018_12',
 'tcsoa.gen.Internal_Core._2020_01',
 'tcsoa.gen.Internal_Core._2020_04',
 'tcsoa.gen.Internal_DebugMonitor',
 'tcsoa.gen.Internal_DebugMonitor._2011_06',
 'tcsoa.gen.Internal_DebugMonitor._2014_06',
 'tcsoa.gen.Internal_DebugMonitor._2015_07',
 'tcsoa.gen.Internal_DebugMonitor._2015_10',
 'tcsoa.gen.Internal_DebugMonitor._2019_06',
 'tcsoa.gen.Internal_DocMgmtAw',
 'tcsoa.gen.Internal_DocMgmtAw._2019_06',
 'tcsoa.gen.Internal_DocMgmtAw._2019_12',
 'tcsoa.gen.Internal_DocumentManagement',
 'tcsoa.gen.Internal_DocumentManagement._2008_06',
 'tcsoa.gen.Internal_DocumentManagement._2013_05',
 'tcsoa.gen.Internal_DocumentManagement._2013_12',
 'tcsoa.gen.Internal_DocumentManagement._2020_12',
 'tcsoa.gen.Internal_EntCba',
 'tcsoa.gen.Internal_EntCba._2019_12',
 'tcsoa.gen.Internal_Gdis',
 'tcsoa.gen.Internal_Gdis._2006_03',
 'tcsoa.gen.Internal_GlobalMultiSite',
 'tcsoa.gen.Internal_GlobalMultiSite._2007_06',
 'tcsoa.gen.Internal_GlobalMultiSite._2008_06',
 'tcsoa.gen.Internal_GlobalMultiSite._2010_02',
 'tcsoa.gen.Internal_GlobalMultiSite._2010_09',
 'tcsoa.gen.Internal_GlobalMultiSite._2017_05',
 'tcsoa.gen.Internal_GlobalMultiSite._2018_11',
 'tcsoa.gen.Internal_GlobalMultiSite._2020_01',
 'tcsoa.gen.Internal_IcsAw',
 'tcsoa.gen.Internal_IcsAw._2017_06',
 'tcsoa.gen.Internal_IcsAw._2017_12',
 'tcsoa.gen.Internal_IcsAw._2018_05',
 'tcsoa.gen.Internal_IcsAw._2018_12',
 'tcsoa.gen.Internal_IcsAw._2019_06',
 'tcsoa.gen.Internal_IcsAw._2019_12',
 'tcsoa.gen.Internal_ImportExport',
 'tcsoa.gen.Internal_ImportExport._2010_04',
 'tcsoa.gen.Internal_ImportExport._2017_05',
 'tcsoa.gen.Internal_ImportExport._2017_11',
 'tcsoa.gen.Internal_ImportExport._2018_06',
 'tcsoa.gen.Internal_Integration',
 'tcsoa.gen.Internal_Integration._2007_06',
 'tcsoa.gen.Internal_Integration._2008_06',
 'tcsoa.gen.Internal_Manufacturing',
 'tcsoa.gen.Internal_Manufacturing._2008_12',
 'tcsoa.gen.Internal_Manufacturing._2011_12',
 'tcsoa.gen.Internal_Manufacturing._2012_09',
 'tcsoa.gen.Internal_Manufacturing._2013_12',
 'tcsoa.gen.Internal_Manufacturing._2014_06',
 'tcsoa.gen.Internal_Manufacturing._2014_12',
 'tcsoa.gen.Internal_Manufacturing._2015_03',
 'tcsoa.gen.Internal_Manufacturing._2015_10',
 'tcsoa.gen.Internal_Manufacturing._2016_03',
 'tcsoa.gen.Internal_Manufacturing._2016_09',
 'tcsoa.gen.Internal_Manufacturing._2017_05',
 'tcsoa.gen.Internal_Manufacturing._2017_11',
 'tcsoa.gen.Internal_Manufacturing._2018_11',
 'tcsoa.gen.Internal_Manufacturing._2019_06',
 'tcsoa.gen.Internal_Manufacturing._2020_01',
 'tcsoa.gen.Internal_Manufacturing._2020_04',
 'tcsoa.gen.Internal_Mmv',
 'tcsoa.gen.Internal_Mmv._2012_09',
 'tcsoa.gen.Internal_MultiSite',
 'tcsoa.gen.Internal_MultiSite._2007_06',
 'tcsoa.gen.Internal_MultiSite._2011_06',
 'tcsoa.gen.Internal_MultiSite._2012_02',
 'tcsoa.gen.Internal_MultiSite._2017_11',
 'tcsoa.gen.Internal_MultiSite._2018_06',
 'tcsoa.gen.Internal_MultiSite._2020_12',
 'tcsoa.gen.Internal_Notification',
 'tcsoa.gen.Internal_Notification._2014_10',
 'tcsoa.gen.Internal_Notification._2015_03',
 'tcsoa.gen.Internal_Notification._2015_10',
 'tcsoa.gen.Internal_Notification._2017_11',
 'tcsoa.gen.Internal_OccMgmt',
 'tcsoa.gen.Internal_OccMgmt._2020_05',
 'tcsoa.gen.Internal_OccMgmt._2020_12',
 'tcsoa.gen.Internal_ProductConfiguratorAw',
 'tcsoa.gen.Internal_ProductConfiguratorAw._2017_12',
 'tcsoa.gen.Internal_ProductConfiguratorAw._2018_05',
 'tcsoa.gen.Internal_ProductConfiguratorAw._2020_05',
 'tcsoa.gen.Internal_ProductConfiguratorAw._2020_12',
 'tcsoa.gen.Internal_ProjectManagement',
 'tcsoa.gen.Internal_ProjectManagement._2007_01',
 'tcsoa.gen.Internal_ProjectManagement._2007_06',
 'tcsoa.gen.Internal_ProjectManagement._2008_06',
 'tcsoa.gen.Internal_ProjectManagement._2009_10',
 'tcsoa.gen.Internal_ProjectManagement._2010_04',
 'tcsoa.gen.Internal_ProjectManagement._2011_06',
 'tcsoa.gen.Internal_ProjectManagement._2012_02',
 'tcsoa.gen.Internal_ProjectManagement._2014_10',
 'tcsoa.gen.Internal_ProjectManagementAw',
 'tcsoa.gen.Internal_ProjectManagementAw._2019_12',
 'tcsoa.gen.Internal_Query',
 'tcsoa.gen.Internal_Query._2008_06',
 'tcsoa.gen.Internal_Query._2012_02',
 'tcsoa.gen.Internal_Query._2013_05',
 'tcsoa.gen.Internal_Query._2014_10',
 'tcsoa.gen.Internal_Rdv',
 'tcsoa.gen.Internal_Rdv._2007_06',
 'tcsoa.gen.Internal_Rdv._2007_09',
 'tcsoa.gen.Internal_Rdv._2008_03',
 'tcsoa.gen.Internal_Rdv._2009_01',
 'tcsoa.gen.Internal_Rdv._2009_04',
 'tcsoa.gen.Internal_Rdv._2009_05',
 'tcsoa.gen.Internal_Rdv._2010_04',
 'tcsoa.gen.Internal_Reports',
 'tcsoa.gen.Internal_Reports._2007_06',
 'tcsoa.gen.Internal_Requirementsmanagement',
 'tcsoa.gen.Internal_Requirementsmanagement._2009_10',
 'tcsoa.gen.Internal_Requirementsmanagement._2012_02',
 'tcsoa.gen.Internal_Requirementsmanagement._2012_10',
 'tcsoa.gen.Internal_RevRuleMgmt',
 'tcsoa.gen.Internal_RevRuleMgmt._2019_12',
 'tcsoa.gen.Internal_Search',
 'tcsoa.gen.Internal_Search._2020_12',
 'tcsoa.gen.Internal_Security',
 'tcsoa.gen.Internal_Security._2017_12',
 'tcsoa.gen.Internal_Structure',
 'tcsoa.gen.Internal_Structure._2020_12',
 'tcsoa.gen.Internal_StructureManagement',
 'tcsoa.gen.Internal_StructureManagement._2007_06',
 'tcsoa.gen.Internal_StructureManagement._2007_12',
 'tcsoa.gen.Internal_StructureManagement._2008_03',
 'tcsoa.gen.Internal_StructureManagement._2008_05',
 'tcsoa.gen.Internal_StructureManagement._2008_06',
 'tcsoa.gen.Internal_StructureManagement._2009_10',
 'tcsoa.gen.Internal_StructureManagement._2010_04',
 'tcsoa.gen.Internal_StructureManagement._2010_09',
 'tcsoa.gen.Internal_StructureManagement._2011_06',
 'tcsoa.gen.Internal_StructureManagement._2012_02',
 'tcsoa.gen.Internal_StructureManagement._2012_09',
 'tcsoa.gen.Internal_StructureManagement._2013_05',
 'tcsoa.gen.Internal_StructureManagement._2013_12',
 'tcsoa.gen.Internal_StructureManagement._2014_12',
 'tcsoa.gen.Internal_StructureManagement._2015_10',
 'tcsoa.gen.Internal_StructureManagement._2016_03',
 'tcsoa.gen.Internal_StructureManagement._2016_10',
 'tcsoa.gen.Internal_StructureManagement._2017_05',
 'tcsoa.gen.Internal_StructureManagement._2018_11',
 'tcsoa.gen.Internal_StructureManagement._2019_06',
 'tcsoa.gen.Internal_StructureManagement._2020_05',
 'tcsoa.gen.Internal_TCXMLImportExport',
 'tcsoa.gen.Internal_TCXMLImportExport._2020_12',
 'tcsoa.gen.Internal_Translation',
 'tcsoa.gen.Internal_Translation._2007_06',
 'tcsoa.gen.Internal_UiConfig',
 'tcsoa.gen.Internal_UiConfig._2014_11',
 'tcsoa.gen.Internal_Validation',
 'tcsoa.gen.Internal_Validation._2007_06',
 'tcsoa.gen.Internal_Validation._2008_06',
 'tcsoa.gen.Internal_Validation._2010_04',
 'tcsoa.gen.Internal_Validation._2012_02',
 'tcsoa.gen.Internal_VendorManagementAW',
 'tcsoa.gen.Internal_VendorManagementAW._2019_12',
 'tcsoa.gen.Internal_VendorManagementAW._2020_05',
 'tcsoa.gen.Internal_VendorManagementAW._2020_12',
 'tcsoa.gen.Internal_Vendormanagement',
 'tcsoa.gen.Internal_Vendormanagement._2008_05',
 'tcsoa.gen.Internal_Vendormanagement._2008_06',
 'tcsoa.gen.Internal_Visualization',
 'tcsoa.gen.Internal_Visualization._2008_06',
 'tcsoa.gen.Internal_Visualization._2010_04',
 'tcsoa.gen.Internal_Visualization._2010_09',
 'tcsoa.gen.Internal_Visualization._2011_12',
 'tcsoa.gen.Internal_Visualization._2012_10',
 'tcsoa.gen.Internal_Visualization._2017_05',
 'tcsoa.gen.Internal_Visualization._2018_11',
 'tcsoa.gen.Internal_Workflow',
 'tcsoa.gen.Internal_Workflow._2013_05',
 'tcsoa.gen.Internal_Workflow._2017_11',
 'tcsoa.gen.Internal_Workflowaw',
 'tcsoa.gen.Internal_Workflowaw._2020_12',
 'tcsoa.gen.Internal_XlsBom',
 'tcsoa.gen.Internal_XlsBom._2020_12',
 'tcsoa.gen.Manufacturing',
 'tcsoa.gen.Manufacturing._2008_06',
 'tcsoa.gen.Manufacturing._2008_12',
 'tcsoa.gen.Manufacturing._2009_06',
 'tcsoa.gen.Manufacturing._2009_10',
 'tcsoa.gen.Manufacturing._2010_09',
 'tcsoa.gen.Manufacturing._2011_06',
 'tcsoa.gen.Manufacturing._2012_02',
 'tcsoa.gen.Manufacturing._2012_09',
 'tcsoa.gen.Manufacturing._2013_05',
 'tcsoa.gen.Manufacturing._2013_12',
 'tcsoa.gen.Manufacturing._2014_06',
 'tcsoa.gen.Manufacturing._2014_12',
 'tcsoa.gen.Manufacturing._2015_03',
 'tcsoa.gen.Manufacturing._2015_10',
 'tcsoa.gen.Manufacturing._2016_03',
 'tcsoa.gen.Manufacturing._2017_05',
 'tcsoa.gen.Manufacturing._2017_11',
 'tcsoa.gen.Manufacturing._2018_06',
 'tcsoa.gen.Manufacturing._2018_11',
 'tcsoa.gen.Manufacturing._2019_06',
 'tcsoa.gen.Multisite',
 'tcsoa.gen.Multisite._2007_06',
 'tcsoa.gen.Multisite._2011_06',
 'tcsoa.gen.Multisite._2014_10',
 'tcsoa.gen.Multisite._2019_06',
 'tcsoa.gen.Notification',
 'tcsoa.gen.Notification._2014_10',
 'tcsoa.gen.OfficeOnline',
 'tcsoa.gen.OfficeOnline._2017_11',
 'tcsoa.gen.Participant',
 'tcsoa.gen.Participant._2018_11',
 'tcsoa.gen.ProjectManagement',
 'tcsoa.gen.ProjectManagement._2007_01',
 'tcsoa.gen.ProjectManagement._2007_06',
 'tcsoa.gen.ProjectManagement._2007_12',
 'tcsoa.gen.ProjectManagement._2008_06',
 'tcsoa.gen.ProjectManagement._2009_10',
 'tcsoa.gen.ProjectManagement._2011_02',
 'tcsoa.gen.ProjectManagement._2011_06',
 'tcsoa.gen.ProjectManagement._2011_12',
 'tcsoa.gen.ProjectManagement._2012_02',
 'tcsoa.gen.ProjectManagement._2012_09',
 'tcsoa.gen.ProjectManagement._2014_06',
 'tcsoa.gen.ProjectManagement._2014_10',
 'tcsoa.gen.ProjectManagement._2015_07',
 'tcsoa.gen.ProjectManagement._2016_04',
 'tcsoa.gen.ProjectManagement._2017_11',
 'tcsoa.gen.ProjectManagement._2018_11',
 'tcsoa.gen.ProjectManagementAw',
 'tcsoa.gen.ProjectManagementAw._2015_07',
 'tcsoa.gen.ProjectManagementAw._2016_12',
 'tcsoa.gen.ProjectManagementAw._2017_06',
 'tcsoa.gen.ProjectManagementAw._2018_12',
 'tcsoa.gen.ProjectManagementAw._2019_06',
 'tcsoa.gen.ProjectManagementAw._2020_05',
 'tcsoa.gen.Qualification',
 'tcsoa.gen.Qualification._2014_06',
 'tcsoa.gen.Query',
 'tcsoa.gen.Query._2006_03',
 'tcsoa.gen.Query._2007_01',
 'tcsoa.gen.Query._2007_06',
 'tcsoa.gen.Query._2007_09',
 'tcsoa.gen.Query._2008_06',
 'tcsoa.gen.Query._2010_04',
 'tcsoa.gen.Query._2010_09',
 'tcsoa.gen.Query._2012_10',
 'tcsoa.gen.Query._2013_05',
 'tcsoa.gen.Query._2014_11',
 'tcsoa.gen.Query._2018_11',
 'tcsoa.gen.Query._2019_06',
 'tcsoa.gen.Query._2020_04',
 'tcsoa.gen.Rdv',
 'tcsoa.gen.Rdv._2008_05',
 'tcsoa.gen.Rdv._2009_04',
 'tcsoa.gen.Rdv._2010_09',
 'tcsoa.gen.Rdv._2012_09',
 'tcsoa.gen.Rdv._2013_05',
 'tcsoa.gen.RelationshipViewer',
 'tcsoa.gen.RelationshipViewer._2012_10',
 'tcsoa.gen.RelationshipViewer._2014_11',
 'tcsoa.gen.RelationshipViewer._2019_12',
 'tcsoa.gen.Reports',
 'tcsoa.gen.Reports._2007_01',
 'tcsoa.gen.Reports._2007_06',
 'tcsoa.gen.Reports._2008_06',
 'tcsoa.gen.Reports._2008_12',
 'tcsoa.gen.Reports._2010_04',
 'tcsoa.gen.Reports._2015_03',
 'tcsoa.gen.Reports._2015_10',
 'tcsoa.gen.Requirementsmanagement',
 'tcsoa.gen.Requirementsmanagement._2007_01',
 'tcsoa.gen.Requirementsmanagement._2008_06',
 'tcsoa.gen.Requirementsmanagement._2009_10',
 'tcsoa.gen.Requirementsmanagement._2010_09',
 'tcsoa.gen.Requirementsmanagement._2011_06',
 'tcsoa.gen.Requirementsmanagement._2012_09',
 'tcsoa.gen.StructureManagement',
 'tcsoa.gen.StructureManagement._2007_06',
 'tcsoa.gen.StructureManagement._2008_03',
 'tcsoa.gen.StructureManagement._2008_05',
 'tcsoa.gen.StructureManagement._2008_06',
 'tcsoa.gen.StructureManagement._2008_12',
 'tcsoa.gen.StructureManagement._2010_04',
 'tcsoa.gen.StructureManagement._2010_09',
 'tcsoa.gen.StructureManagement._2011_06',
 'tcsoa.gen.StructureManagement._2012_02',
 'tcsoa.gen.StructureManagement._2012_09',
 'tcsoa.gen.StructureManagement._2012_10',
 'tcsoa.gen.StructureManagement._2013_05',
 'tcsoa.gen.StructureManagement._2014_06',
 'tcsoa.gen.StructureManagement._2014_10',
 'tcsoa.gen.StructureManagement._2014_12',
 'tcsoa.gen.StructureManagement._2015_10',
 'tcsoa.gen.StructureManagement._2016_05',
 'tcsoa.gen.StructureManagement._2016_09',
 'tcsoa.gen.StructureManagement._2017_05',
 'tcsoa.gen.StructureManagement._2018_11',
 'tcsoa.gen.StructureManagement._2019_06',
 'tcsoa.gen.Translation',
 'tcsoa.gen.Translation._2007_06',
 'tcsoa.gen.UiConfig',
 'tcsoa.gen.UiConfig._2014_11',
 'tcsoa.gen.UiConfig._2015_10',
 'tcsoa.gen.Vendormanagement',
 'tcsoa.gen.Vendormanagement._2007_06',
 'tcsoa.gen.Vendormanagement._2008_06',
 'tcsoa.gen.Vendormanagement._2012_02',
 'tcsoa.gen.Vendormanagement._2012_09',
 'tcsoa.gen.Vendormanagement._2016_09',
 'tcsoa.gen.Visualization',
 'tcsoa.gen.Visualization._2011_02',
 'tcsoa.gen.Visualization._2013_05',
 'tcsoa.gen.Visualization._2013_12',
 'tcsoa.gen.Visualization._2016_03',
 'tcsoa.gen.WProxy',
 'tcsoa.gen.WProxy._2014_10',
 'tcsoa.gen.Workflow',
 'tcsoa.gen.Workflow._2007_06',
 'tcsoa.gen.Workflow._2008_06',
 'tcsoa.gen.Workflow._2010_09',
 'tcsoa.gen.Workflow._2012_10',
 'tcsoa.gen.Workflow._2013_05',
 'tcsoa.gen.Workflow._2014_06',
 'tcsoa.gen.Workflow._2014_10',
 'tcsoa.gen.Workflow._2015_07',
 'tcsoa.gen.Workflow._2019_06',
 'tcsoa.gen.Workflow._2020_01']

package_data = \
{'': ['*']}

install_requires = \
['Deprecated>=1.2.13,<2.0.0', 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'tcsoa',
    'version': '0.9.0',
    'description': 'A framework generated for the purpose of consuming Teamcenter™ services via Python.',
    'long_description': 'None',
    'author': 'Alexander Haas',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject, Folder, Dataset, Item, ItemRevision
from tcsoa.gen.Cad._2007_01.DataManagement import RelationAndTypesFilter2, ExtraObjectOutput, ExpandFoldersForCADItemOutput, ExpandFoldersForCADItemRevOutput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ExpandFolderForCADPref2(TcBaseObj):
    """
    Contains a list of 'RelationAndTypesFilter', number of latest revisions for further filtering and a flag to check
    whether item revision needs to be expanded and list of other object types to return from the folder contents.
    
    :var expItemRev: Flag to specify if the datasets attached to the item revisions are to be returned. If set to TRUE
    then all datasets owned by each item revision will be returned with the item revision.
    :var latestNRevs: The number of revisions under an item that should be considered for further filtering. If there
    are 5 versions of an item and this option is set to 2 then versions 4 and 5 will be processed. 0 = do not return
    any item revisions. minus 1 = return all item revisions.
    :var info: A list of 'RelationAndTypesFilter2' structures to select the datasets under an item revision to return.
    If empty then all dataset will be returned.
    :var contentTypesFilter: A list of object types. Any objects in any input folder that are of the type in this list
    will also be returned.
    """
    expItemRev: bool = False
    latestNRevs: int = 0
    info: List[RelationAndTypesFilter2] = ()
    contentTypesFilter: List[str] = ()


@dataclass
class ExpandFoldersForCADOutput2(TcBaseObj):
    """
    Contains the folder expanded and the results of expanding the folder based on the input preference.
    
    :var inputFolder: The folder that was expanded.
    :var fstlvlFolders: A list of folders contained by the input folder.
    :var itemsOutput: A list of 'ExpandFolderForCADItemOutput' which has information about the items in the folder.
    :var itemRevsOutput: A list of 'ExpandFolderForCADItemRevOutput' which has information about the item revisions in
    the folder.
    :var contents: A list of objects in the folder that are a type listed in the 'contentTypesFilter' input.
    """
    inputFolder: Folder = None
    fstlvlFolders: List[Folder] = ()
    itemsOutput: List[ExpandFoldersForCADItemOutput] = ()
    itemRevsOutput: List[ExpandFoldersForCADItemRevOutput] = ()
    contents: List[WorkspaceObject] = ()


@dataclass
class ExpandFoldersForCADResponse2(TcBaseObj):
    """
    Contains the response for 'ExpandFoldersForCAD' operation.
    
    :var output: A list of 'ExpandFoldersForCADOutput2' which has information about the input folder and folders, items
    and item revisions output found under this folder
    :var serviceData: Contains the folders, items, items revisions, datasets, and other object types in the list of
    plain objects. 
    Error information returned is identified by the folder tag sent to the service and will include system errors plus
    errors generated by this service :
       7007 : Input folder is invalid.
    215121 : Object passed in is not a folder.
    """
    output: List[ExpandFoldersForCADOutput2] = ()
    serviceData: ServiceData = None


@dataclass
class ExtraObjectInfo(TcBaseObj):
    """
    Form objects that can be created or updated and related to an item, item revision or dataset.
    
    :var object: Object reference for existing object
    :var clientId: Identifier defined by user to track the related object.
    :var relationTypeName: Name of the relation type to the parent object
    :var typeName: Object Type name
    :var attrNameValuePairs: List of AttributeInfos.
    """
    object: BusinessObject = None
    clientId: str = ''
    relationTypeName: str = ''
    typeName: str = ''
    attrNameValuePairs: List[AttributeInfo] = ()


@dataclass
class ItemInfo(TcBaseObj):
    """
    the ItemInfo structure represents all of the data necessary to construct the item object.
    The basic attributes that are required are passed as named elements in the structure.
    All other attributes are passed as name/value pairs in the AttributeInfo structure.
    The extraObject field allows for the creation of an object(s) that will be related to this newly created Item.
    
    :var item: Item object reference for update, can be null for creation
    :var itemId: ID for create, generated if null
    :var itemType: Type, default is Item if null
    :var name: Name, defaulted to id if null
    :var description: Description, can be null
    :var attrList: List of AttributeInfos
    :var extraObject: List of ExtraObjectInfos
    :var folder: Folder to attach Item to, if null then default used
    """
    item: Item = None
    itemId: str = ''
    itemType: str = ''
    name: str = ''
    description: str = ''
    attrList: List[AttributeInfo] = ()
    extraObject: List[ExtraObjectInfo] = ()
    folder: Folder = None


@dataclass
class ItemRevInfo(TcBaseObj):
    """
    the ItemRevInfo structure represents all of the data necessary to construct the item revision object.
    The basic attributes that are required are passed as named elements in the struct.
    All other attributes are passed as name/value pairs in the AttributeInfo structure.
    The extraObject field allows for the creation of an object(s) that will be related to this newly created Item
    Revision.
    
    :var itemRevision: ItemRevision object reference, null for creation, otherwise update
    :var revId: ID, if null then generated
    :var attrList: List of AttributeInfos
    :var extraObject: List ofr ExtraObjectInfos
    """
    itemRevision: ItemRevision = None
    revId: str = ''
    attrList: List[AttributeInfo] = ()
    extraObject: List[ExtraObjectInfo] = ()


@dataclass
class NamedReferenceObjectInfo(TcBaseObj):
    """
    Contains information regarding named reference type value, object reference, object type name and list of attribute
    information to set on the object.
    
    :var clientId: Identifier defined by client to track the related object.
    :var object: Object reference of the object for update, null for create.
    :var namedReferenceName: The Named Reference from the dataset to this object, required. NamedReference values  are
    defined for each Dataset type. The customer can add more values as needed. To get a current list of valid Named
    Reference values the programmer can either use BMIDE or can call the SOA Core service getDatasetTypeIno.
    :var namedReferenceType: The reference type name from the dataset to this object, must be either AE_ASSOCIATION or
    AE_PART_OF.
    :var typeName: Type of the object to be created. Required for object creation only.
    :var attrNameValuePairs: List of attribute information.
    """
    clientId: str = ''
    object: BusinessObject = None
    namedReferenceName: str = ''
    namedReferenceType: str = ''
    typeName: str = ''
    attrNameValuePairs: List[AttributeInfo] = ()


@dataclass
class AttributeInfo(TcBaseObj):
    """
    This structure allows the caller define or update named attributes.
    The name member represents the property name for the related object and the value is the value to set.
    
    :var name: Text for attribute name
    :var values: Text for attribute value
    """
    name: str = ''
    values: List[str] = ()


@dataclass
class PartInfo(TcBaseObj):
    """
    The PartInfo struct is the main input to the createOrUpdateParts service for boundingbox. This structure refers to
    the Item, ItemRevision, and one or more Dataset structures used to create those objects.
    
    :var clientId: Identifier defined by user to track the related object.
    :var itemInput: Member of type ItemInfo
    :var itemRevInput: Member of type ItemRevInfo
    :var datasetInput: List of DatasetInfos
    """
    clientId: str = ''
    itemInput: ItemInfo = None
    itemRevInput: ItemRevInfo = None
    datasetInput: List[DatasetInfo] = ()


@dataclass
class BoundingBox(TcBaseObj):
    """
    Holds the boundingbox co-ordinates  information.
    
    :var xmin: BoundingBox x-coordinate min value in double
    :var ymin: BoundingBox y-coordinate min value in double
    :var zmin: BoundingBox z-coordinate min value in double
    :var xmax: BoundingBox x-coordinate max value in double
    :var ymax: BoundingBox y-coordinate max value in double
    :var zmax: BoundingBox z-coordinate max value in double
    """
    xmin: float = 0.0
    ymin: float = 0.0
    zmin: float = 0.0
    xmax: float = 0.0
    ymax: float = 0.0
    zmax: float = 0.0


@dataclass
class CommitDatasetFileInfo(TcBaseObj):
    """
    Holds the basic info for a file to be uploaded to a dataset.
    
    :var dataset: 'Dataset' object reference.
    :var createNewVersion: Flag to create new version ( TRUE ) or not ( FALSE ).
    :var datasetFileTicketInfos: A list of 'DatasetFileTicketInfos' which contains file ticket information for the
    dataset file.
    """
    dataset: Dataset = None
    createNewVersion: bool = False
    datasetFileTicketInfos: List[DatasetFileTicketInfo] = ()


@dataclass
class CreateOrUpdatePartsOutput(TcBaseObj):
    """
    Contains the output response structure for createOrUpdateParts operation.
    
    :var clientId: Identifier defined by user to track the related object.
    :var item: Item object reference of the created/updated item
    :var itemRev: ItemRevision object reference of the created/updated item revision
    :var datasetOutput: List of DatasetOutputs
    :var extraItemObjs: List of ExtraObjectOutputs for the item extra objects
    :var extraItemRevObjs: List of ExtraObjectOutputs for the item revision extra objects
    """
    clientId: str = ''
    item: Item = None
    itemRev: ItemRevision = None
    datasetOutput: List[DatasetOutput] = ()
    extraItemObjs: List[ExtraObjectOutput] = ()
    extraItemRevObjs: List[ExtraObjectOutput] = ()


@dataclass
class CreateOrUpdatePartsResponse(TcBaseObj):
    """
    Holds the response for createOrUpdateParts
    
    :var output: List of CreateOrUpdatePartsOutputs.
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error information.
    """
    output: List[CreateOrUpdatePartsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DatasetFileInfo(TcBaseObj):
    """
    Holds the basic info for a file to be uploaded to a dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var fileName: Name of file to be uploaded.  Filename only, should not contain path to filename.
    :var namedReferencedName: Named Reference relation to file.
    :var isText: Flag to indicate if file is text ( TRUE ) or binary (FALSE ).
    :var allowReplace: Flag to indicate if file can be overwritten ( TRUE ) or not ( FALSE ).
    :var boundingBoxesAvailable: Flag to indicate BoundingBoxes are available.
    :var boundingBoxes: List of BoundingBoxes.
    """
    clientId: str = ''
    fileName: str = ''
    namedReferencedName: str = ''
    isText: bool = False
    allowReplace: bool = False
    boundingBoxesAvailable: bool = False
    boundingBoxes: List[BoundingBox] = ()


@dataclass
class DatasetFileTicketInfo(TcBaseObj):
    """
    Holds the basic info for a file to be uploaded to a dataset.
    
    :var datasetFileInfo: Member of type DatasetFileInfo.
    :var ticket: ID of ticket.
    """
    datasetFileInfo: DatasetFileInfo = None
    ticket: str = ''


@dataclass
class DatasetInfo(TcBaseObj):
    """
    The DatasetInfo struct represents all of the data necessary to construct the dataset object.
    The basic attributes that are required are passed as named elements in the structure.
    All other attributes are passed as name/value pairs in the AttributeInfo structure.
    The extraObject field allows for the creation of an object(s) that will be related to this newly created Dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var dataset: Dataset object reference for update, null for creation
    :var createNewVersion: Flag to create new version ( TRUE ) or not (FALSE )
    :var mapAttributesWithoutDataset: Flag to indicate whether DatasetInfo should be used for mapping attributes or for
    create.
    :var namedReferencePreference: Preference name which holds the list of named references to delete from one Dataset
    version to the next
    :var attrList: List of AttributeInfos for attributes
    :var mappingAttributes: List of AttributeInfos for mapped attributes. Mapped atributes are attributes that are
    applied to other objects. Refere to the ITK manual for a definition of attribute mapping.
    :var extraObject: List of ExtraObjectInfos
    :var datasetFileInfos: List of DatasetFileInfos
    :var namedReferenceObjectInfos: List of NamedReferenceObjectInfos
    :var name: Name attribute value
    :var basisName: basisName
    :var description: Description attribute value
    :var type: Type attribute value
    :var lastModifiedOfDataset: lastModifiedOfDataset
    :var id: ID attribute value
    :var datasetRev: Revision attribute value
    :var itemRevRelationName: Can be null, defaulted
    """
    clientId: str = ''
    dataset: Dataset = None
    createNewVersion: bool = False
    mapAttributesWithoutDataset: bool = False
    namedReferencePreference: str = ''
    attrList: List[AttributeInfo] = ()
    mappingAttributes: List[AttributeInfo] = ()
    extraObject: List[ExtraObjectInfo] = ()
    datasetFileInfos: List[DatasetFileInfo] = ()
    namedReferenceObjectInfos: List[NamedReferenceObjectInfo] = ()
    name: str = ''
    basisName: str = ''
    description: str = ''
    type: str = ''
    lastModifiedOfDataset: datetime = None
    id: str = ''
    datasetRev: str = ''
    itemRevRelationName: str = ''


@dataclass
class DatasetOutput(TcBaseObj):
    """
    Contains created/updated dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var dataset: Dataset object reference of the created/updated dataset
    :var commitInfo: List of 'CommitDatasetFileInfos'
    :var extraObjs: List of 'ExtraObjectOutputs' for the extra objects
    :var namedRefObjs: List of 'ExtraObjectOutputs' for the named references
    """
    clientId: str = ''
    dataset: Dataset = None
    commitInfo: List[CommitDatasetFileInfo] = ()
    extraObjs: List[ExtraObjectOutput] = ()
    namedRefObjs: List[ExtraObjectOutput] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class DeepCopyData(TcBaseObj):
    """
    DeepCopyData stores the deep copy information that will be copied via derive operation. It also stores the nested
    deep copy data at the next level.
    
    :var attachedObject: The related object to be deep copied.
    :var deepCopyProperties: deepCopyProperties.
    :var operationInputType: Object type name of the operation being perfomed.
    :var childDeepCopyData: A list of DeepCopyData for the objects of the relation or reference property objects of the
    attached object.
    :var inputProperties:  Map to provide input property names and values of attached object. These property values
    will be applied on propagated objects. Map of property name(key) and property values(values) (string, list of
    strings) in string format of attached object, to be set on copied object of attached object. The calling client is
    responsible for converting the different property types (int, float, date .etc) to a string using the appropriate
    toXXXString functions in the SOA client framework Property class.
    """
    attachedObject: BusinessObject = None
    deepCopyProperties: PropertyValueMap = None
    operationInputType: str = ''
    childDeepCopyData: List[DeepCopyData] = ()
    inputProperties: PropertyValueMap = None


@dataclass
class DeriveInput(TcBaseObj):
    """
    Structure that contains multiple Change Items object to be derived, a structure containing property name and values
    to be propagated,  Deep Copy Data, a flag to automatically submit to workflow or not, a workflow template name and
    a flag to propagate relations or not.
    
    :var selectedObjects: A list of Problem Report or Change Request objects to be derived.
    :var derivePropertyData: A map of property name (as key) and property values (as value) in string format. Each
    value is a list of strings to support both single valued and multi valued properties of types. The calling client
    is responsible for converting the different property types (like integer, double, date, etc) to a string using the
    appropriate to< type >String function (e.g. toIntString and toDateString) in the client framework's Property class.
    :var deepCopyDatas: A list of DeepCopyData to be propagated to the derived Change Item objects.
    :var submitToWorkflow: This is used to indicate whether the derived Change Item object will be automatically
    submitted to Workflow. If true will submit the derived object to designated workflow template, otherwise the
    derived object will not be submitted to workflow.
    :var workflowTemplateName: The workflow template name to be executed, if not informed, the default workflow
    template will be used.
    :var propagateRelation: If TRUE will propagate the relation defined in Deep Copy Rule, if FALSE will not propagate
    the relations.
    """
    selectedObjects: List[BusinessObject] = ()
    derivePropertyData: DerivePropertyValueInput = None
    deepCopyDatas: List[DeepCopyData] = ()
    submitToWorkflow: bool = False
    workflowTemplateName: str = ''
    propagateRelation: bool = False


@dataclass
class DerivePropertyValueInput(TcBaseObj):
    """
    This map is of property name (as key) and a vector of property values input, business object in string format,
    property name values map and compund derive input. The business object is a simple string. The property name values
    is a list of strings to support both single valued and multi valued properties of types. The calling client is
    responsible for converting the different property types (like integer, double, date, etc) to a string using the
    appropriate to< type >String function (e.g. toIntString and toDateString) in the client framework's Property class.
    The compound derive input is another type of def of property value input data structure.
    
    :var boName: The buisness object name to which the properties belong.
    :var propertyNameValues: Map of property name values.
    :var compoundDeriveInput: Vector of derive property value input.
    """
    boName: str = ''
    propertyNameValues: PropertyValueMap = None
    compoundDeriveInput: DerivePropertyValueInputMap = None


"""
This map is of derive property value input with a string (as key) and a vector of derive property values input consisting of a business object in string format, a property name values map and compund input of derive property values.
"""
DerivePropertyValueInputMap = Dict[str, List[DerivePropertyValueInput]]


"""
This map is of property name (as key) and property values (as value) in string format. Each value is a list of strings to support both single valued and multi valued properties of types. The calling client is responsible for converting the different property types (like integer, double, date, etc) to a string using the appropriate to< type >String function (e.g. toIntString and toDateString) in the client framework's Property class
"""
PropertyValueMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BusinessObject
from typing import List
from tcsoa.gen.Internal.StructureManagement._2012_02.StructureVerification import PartialMatchCriteria2
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindReviewStatusDetails(TcBaseObj):
    """
    The review status details for a list of impacted target scopes.
    
    :var inputIndex: Corresponds to the index of the input list.
    :var statusDetails: Corresponds to the details of the review status associated with impacted target scopes. This
    list is in parallel with the changedObjects list in FindReviewStatusIn.
    """
    inputIndex: int = 0
    statusDetails: List[FindSingleReviewStatusDetail] = ()


@dataclass
class FindReviewStatusIn(TcBaseObj):
    """
    Input object that contains the changed line, the change identifier, and the impacted target scopes.
    
    :var changedLine: The changed BOMLine object.
    :var changedObjects: The changed objects, which can be IncrementalChangeElement, ItemRevision, or PSOccurrence
    objects.
    :var impactedScopes: A list of BOMLine objects representing the impacted target scopes.
    """
    changedLine: BOMLine = None
    changedObjects: List[BusinessObject] = ()
    impactedScopes: List[BOMLine] = ()


@dataclass
class FindReviewStatusResponse(TcBaseObj):
    """
    The response that contains the review status details for a list of impacted target scopes.
    
    :var statusDetails: The review status details information. Each FindReviewStatusDetails object in the statusDetails
    list represents the status details for a single FindReviewStatusIn object in the input list.
    :var serviceData: Will contain the client ID and partial errors.
    """
    statusDetails: List[FindReviewStatusDetails] = ()
    serviceData: ServiceData = None


@dataclass
class FindSingleReviewStatusDetail(TcBaseObj):
    """
    Review status detail for an impacted target scope.
    
    :var resultantScope: The scope object whose review status is used. It can be any of the following: the top most
    scope object which has a review status, the target scope itself if it has a review status, a child scope if this is
    the only child which has a review status, or null if the target scope has multiple children with a review status.
    :var resultantStatus: The review status from the resultantScope.
    :var evaluatedScopes: In case there is a review status on multiple sub-scopes, this field will contain the
    sub-scope objects.
    :var evaluatedStatus: In case there is a review status on multiple sub-scopes, this field will contain the status
    of each sub-scope object.
    :var setByUser: Indicates if this review status is explicitly set by the user.
    """
    resultantScope: BusinessObject = None
    resultantStatus: str = ''
    evaluatedScopes: List[BusinessObject] = ()
    evaluatedStatus: List[str] = ()
    setByUser: bool = False


@dataclass
class GetPropPropagationDetailsRespElem(TcBaseObj):
    """
    The getPropertyPropagationStatusDetails response element. Contains property details and propagation status details
    for each input line.
    
    :var propagationStatusDetails: A list of review status detail elements.
    
    :var propertyDetails: A list of  property details for each line.
    :var additionalInfo: Future Use
    """
    propagationStatusDetails: List[FindReviewStatusDetails] = ()
    propertyDetails: List[PropertyDetail2] = ()
    additionalInfo: PartialMatchCriteria2 = None


@dataclass
class GetPropPropagationStatusDetailsIn(TcBaseObj):
    """
    A list of changed lines, the reason for the change and a potential list of candidate impacted lines.
    
    :var changedLines: List of changed BOMLine objects.
    :var impactedLines: List of propagation candidate BOMLine objects.
    :var objectsDescribingChange: List of property changed objects, which can be  IncrementalChangeElement, 
    ItemRevision, or 
    PSOccurrence.
    
    :var additionalInfo: Future Use
    """
    changedLines: List[BusinessObject] = ()
    impactedLines: List[BusinessObject] = ()
    objectsDescribingChange: List[BusinessObject] = ()
    additionalInfo: PartialMatchCriteria2 = None


@dataclass
class GetPropPropagationStatusDetailsResp(TcBaseObj):
    """
    List of property and propagation status details for each input line.
    
    :var details: A list of property and propagation status details.
    :var serviceData: Service data capturing partial errors using the input array index as client id.
    """
    details: List[GetPropPropagationDetailsRespElem] = ()
    serviceData: ServiceData = None


@dataclass
class GetStructureChangeDetailsResponse(TcBaseObj):
    """
    The GetStructureChangeDetailsResponse is returned by the getStructureChangeDetails method.  The response is the
    details of a changed line that was changed by IC, revision, occurrence effectivity or time period.  The details
    returned are the value that was requested in the original call to getStructureChangeDetails.  The response has two
    main element that are:
    
    -     serviceData - Service data capturing partial errors using the input array index as client id.
    -     details - list of StructureChangeDetailsResponseElement
    
    
    
    :var serviceData: Service data capturing partial errors using the input array index as client id.
    :var details: The details of a changed value of the configured input line.
    """
    serviceData: ServiceData = None
    details: List[StructureChangeDetailsResponseElement] = ()


@dataclass
class GetStructureChangeImpactedLinesInfo(TcBaseObj):
    """
    The GetStructureChangeImpactedLinesInfo contains impacted lines and associated line info that were found in the
    input target structure.  The impacted lines are lines chosen in the target structure due to changes in a source
    structure.  The type of changes in the source structure that are supported are:
    
    - Incremental changes
    - Revision effectivity changes
    - Occurrence effectivity changes
    - Time period changes
    
    
    
    :var icImpactedLinesInfo: Impacted lines and associated information found in the target structure that were
    impacted in the source structure through incremental change.  This field is a generic map, currently the following
    key/value pairs are returned:
    
    The PartialMatchCriteria is a structure of maps as follows:
    struct PartialMatchCriteria2
    { 
        StringToIntVectorMap2 intMap,
        StringToDblVectorMap2 dblMap,
        StringToStrVectorMap2 strMap,
        StringToObjVectorMap2 objMap,
        StringToDateVectorMap2 dateMap
    };
    
    Elements:
    - intMap map of string to list of integers. 
    - dblMap map of string to list of doubles. 
    - strMap map of string to list of strings. 
    - objMap map of string to list of objects. 
    - dateMap map of string to list of dates 
    
    
    
    strMap entries:
    
    Key: ChangeType
    Value: One of the following case sensitive strings:
    
    - Add
    - Remove
    - Move
    - Property
    - Attachment
    - Predecessor
    
    
    
    Key: AttributeName (Required for Property Change)
    Value: Internal name of affected value
    
    Key: AttributeValue (Required for Property Change)
    Value: Value that changed   (Aligned with AttributeName list)
    
    Key: PredecessorName (Required for Predecessor Change Type)
    Value: name of predecessor
    
    Key:PredecessorSequence (Required for Predecessor Change Type)
    Value: sequence number of predecessor     
    
    objMap entries:
    
    Key: ChangedLine        Value: BOMLine in the source structure that changed
    
    Key: AffectedObject (Required for Attachment Change)
    Value: An object of one of the following types (not a complete list):
    - PSOccurrence
    - Datasets
    - Folders
    - Forms
    - Activities
    - AbsOccData
    
    
    
    End of the details for the icImpactedLinesInfo PartialMatchCriteria2 structure.
    
    
    
    - findImpactedLinesByRevisionEff    Currently not supported
    - findImpactedLinesByOccEff        Currently not supported
    - findImpactedLinesByPeriod        Currently not supported
    
    
    :var revisionImpactedLinesInfo: Impacted lines and associated information found in the target structure that were
    impacted in the source structure through revision effectivity change. Currently not supported.
    :var occEffImpactedLinesInfo: Impacted lines and associated information found in the target structure that were
    impacted in the source structure through occurrence effectivity change.  Currently not supported.
    :var periodImpactedLinesInfo: Impacted lines and associated information found in the target structure that were
    impacted in the source structure through time period change.  Currently not supported.
    """
    icImpactedLinesInfo: PartialMatchCriteria2 = None
    revisionImpactedLinesInfo: PartialMatchCriteria2 = None
    occEffImpactedLinesInfo: PartialMatchCriteria2 = None
    periodImpactedLinesInfo: PartialMatchCriteria2 = None


@dataclass
class GetStructureChangeImpactedLinesResponse(TcBaseObj):
    """
    The GetStructureChangeImpactedLinesResponse is returned by the getStructureChangeImpactedLines method.  The
    response contains partial errors or impacted lines and associated line information.  Impacted lines are lines in a
    target structure that are affected by changes in a source structure. The following errors may be returned:
    - 214522 The End item specified for the Unit Effectivity is invalid.
    - 214523 No incremental change contexts can be found based on search criteria.
    - 214524 Invalid units specified for searching.
    - 214525 Invalid dates specified for searching.
    - 200451 The incremental change criteria are invalid.
    
    
    
    :var impactedLinesInfo: The impacted lines and associated line info found in the input target structure based on
    changes made to the source structure.  Impacted lines are lines in a target structure that are potentially affected
    by changes in a source structure.
    
    The PartialMatchCriteria is a structure of maps as follows:
    struct PartialMatchCriteria2
    { 
        StringToIntVectorMap2 intMap,
        StringToDblVectorMap2 dblMap,
        StringToStrVectorMap2 strMap,
        StringToObjVectorMap2 objMap,
        StringToDateVectorMap2 dateMap
    };
    
    Elements:
    - intMap map of string to list of integers. 
    - dblMap map of string to list of doubles. 
    - strMap map of string to list of strings. 
    - objMap map of string to list of objects. 
    - dateMap map of string to list of dates 
    
    
    
    strMap entries:
    
    Key: ChangeType
    Value: One of the following case sensitive strings:
    
    - Add
    - Remove
    - Move
    - Property
    - Attachment
    - Predecessor
    
    
    
    Key: AttributeName (Required for Property Change)
    Value: Internal name of affected value
    
    Key: AttributeValue (Required for Property Change)
    Value: Value that changed   (Aligned with AttributeName list)
    
    Key: PredecessorName (Required for Predecessor Change Type)
    Value: name of predecessor
    
    Key:PredecessorSequence (Required for Predecessor Change Type)
    Value: sequence number of predecessor     
    
    objMap entries:
    
    Key: ChangedLine        Value: BOMLine in the source structure that changed
    
    Key: AffectedObject (Required for Attachment Change)
    Value: An object of one of the following types (not a complete list):
    - PSOccurrence
    - Datasets
    - Folders
    - Forms
    - Activities
    - AbsOccData
    
    
    
    End of the details for the icImpactedLinesInfo PartialMatchCriteria2 structure.
    
    
    
    - findImpactedLinesByRevisionEff    Currently not supported
    - findImpactedLinesByOccEff        Currently not supported
    - findImpactedLinesByPeriod        Currently not supported
    
    
    :var serviceData: The partial errors indexed by the input vector.
    """
    impactedLinesInfo: List[GetStructureChangeImpactedLinesInfo] = ()
    serviceData: ServiceData = None


@dataclass
class ImpactedLinesCriteria(TcBaseObj):
    """
    This data structure is used by the getImpactedLinesForStructureChange method.  The data structure provides criteria
    for finding impacted lines in the designated target structure due to changes in a related but different structure. 
    The data structure supports four types of structure changes:  
    
    - Incremental change
    - Revision effectivity - Currently not supported
    - Occurrence effectivity - Currently not supported
    - Time period - Currently not supported
    
    
    
    :var findImpactedLinesByICInfo: This structure contains changed lines under incremental change that will be used to
    find equivalent impacted lines in the input target structure.  This element is a generic map that currently
    supports the following name/value pairs:
    
    strMap entries:
    
    Key: ChangeType
    Value: One of the following case sensitive strings:
    
    - Add
    - Remove
    - Move
    - Property
    - Attachment
    - Predecessor
    
    
    
    objMap entries:
    
    Key: ChangedLine
    Value: BOMLine in the source structure that changed
    
    Key: AffectedObject
    Value: An object of one of the following types (not a complete list):
            PSOccurrence
            Datasets
            Folders
            Forms
            Activities
            AbsOccData
    :var findImpactedLinesByRevisionEff: This structure contains changed lines by revision effectivity that will be
    used to find equivalent impacted lines in input target structure.
    :var findImpactedLinesByOccEff: This structure contains changed lines by occurrence effectivity that will be used
    to find equivalent impacted lines in input target structure.
    :var findImpactedLinesByPeriod: This structure contains changed lines by time period that will be used to find
    equivalent impacted lines in input target structure.
    """
    findImpactedLinesByICInfo: PartialMatchCriteria2 = None
    findImpactedLinesByRevisionEff: PartialMatchCriteria2 = None
    findImpactedLinesByOccEff: PartialMatchCriteria2 = None
    findImpactedLinesByPeriod: PartialMatchCriteria2 = None


@dataclass
class PropagationDetail(TcBaseObj):
    """
    The propagation status for a target line.
    
    :var status: The propagation status.
    :var targetContext: The target line. (MEAppearancePathNode)
    """
    status: str = ''
    targetContext: BusinessObject = None


@dataclass
class PropertyDetail2(TcBaseObj):
    """
    Index and list of property details for one element in the input array.
    
    :var index: Index into the input array.
    :var propertyDetails: List of property details
    """
    index: int = 0
    propertyDetails: List[PropertyDetailsElement2] = ()


@dataclass
class PropertyDetailsElement2(TcBaseObj):
    """
    Internal version of PropertyDetailsElement.
    Contains property name and isDifferent elements.
    
    :var propertyName: The property name
    :var isDifferent: Set true if property is different between the two lines
    """
    propertyName: str = ''
    isDifferent: bool = False


@dataclass
class ReviewStatusInfo(TcBaseObj):
    """
    The status to be set on this target scope.
    
    :var impactedScopeLine: The BOMLine object representing an impacted target scope.
    :var status: The status that needs to be set on this impacted target scope.
    """
    impactedScopeLine: BOMLine = None
    status: str = ''


@dataclass
class StructureChangeDetailsElement(TcBaseObj):
    """
    The StructureChangeDetailsElement contains the changed line, the changed value and the configuration information
    for which detailed information is required about the change.  The main elements of the structure are:
    -     searchContext    The search context that was used to find the original changed lines
    -     icChangesInfo    The changed lines found by incremental change critieria
    -     revisionChangesInfo    The changed lines found by revision criteria (Currently not supported)
    -     occEffChangesInfo    The changed lines found by occurrence effectivity (Currently not supported)
    -     periodChangesInfo    The changed lines found by period changes (Currently not supported)
    
    
    
    :var searchContext: The search context that was used to find the original changed lines.
    
    For Vector entries where only one value is needed always use the first vector position.
    Either 'Date' or 'Unit' effectivity entries are required.
    
    The PartialMatchCriteria2 is a structure of maps as follows:
    struct PartialMatchCriteria2
    { 
        StringToIntVectorMap2 intMap,
        StringToDblVectorMap2 dblMap,
        StringToStrVectorMap2 strMap,
        StringToObjVectorMap2 objMap,
        StringToDateVectorMap2 dateMap
    };
    
    Elements:
    intMap map of string to vector or integers. 
    dblMap map of string to vector of doubles. 
    strMap map of string to vector of strings. 
    objMap map of string to vector of objects. 
    dateMap map of string to vector of dates 
    
     
    Entries for the strMap:
    Key: OwningUser        Value: userid as String
    Key: Statuses            Value: A list of statuses - one status for each element of the vector
    Key: ICIntents            Value: A list of IC intents - one for each element of the vector
    
    
    
    Entries for the intMap:
    Key: IsUnitEffectivity            Value: 0 == false, 1 == true
    Key: IsDateEffectivity        Value: 0 == false, 1 == true
    Key: AlreadyReviewedUnit    Value: already reviewed unit number as int
    Key: ToReviewUnit            Value: to review unit number as int
    
    
    Entries for the dateMap:
    Key: AlreadyReviewedDate        Value: date
    Key: ToReviewDate                Value: date
    Key: StatusSinceDate            Value: status since date
    
    
    Entries for the objMap:
    Key :EndItemComp (Required if Unit Effectivity)    Value: End Item of IC Unit effectivity 
    Key: ScopeLines (Required)            Value: Selected BOMLine objects in the original search 
    
        
        
    
    :var icChangesInfo: The changed lines found by incremental change critieria
    The PartialMatchCriteria2 structure of maps has the following entries:
    For vector entries where only one value is needed always use the first vector position.
    Entries for the objMap:
    
    Key: ChangedLine (Required)        Value: The changed BOMLine
    Key: ICRev (Required)            Value: ItemRevision representing the Incremental Change object 
    Key: ICE (Required)            Value: IncrementalChangeElement associated with the changed line.
    
    Key: AffectedObject            Value: An object of one of the following types (not a complete list):
    - PSOccurrence
    - Datasets
    - Folders
    - Forms
    - Activities
    - AbsOccData
    
    
    
    Entries for the strMap:
    Key: ChangeType (Required)        Value: One of the following case sensitive strings:
    - Add
    - Remove
    - Move
    - Property
    - Attachment
    - Predecessor
    
    
     
    Key: HowConfigured                                                    Value: the incremental change how configured
    string
           
    Key:AttributeName (Required for Property Change Type)    Value: internal name of the attribute for
    attribute/property changes
           
    Key:AttributeValue (Required for Property Change Type)    Value: value of the attribute from incremental    change 
          
          
    Key: PredecessorName (Required for Predecessor Change Type)        Value: name of predecessor
    
    Key:PredecessorSequence (Required for Predecessor Change Type)    Value: sequence number of predecessor     
     
    Key:UnitRangeText (Required for unit effectivity)                                Value: unit range text
    
    Key:DateRangeText (Required for date effectivity)                            Value: date range text
    :var revisionChangesInfo: This field is currently not supported.
    :var occEffChangesInfo: This field is currently not supported.
    :var periodChangesInfo: This field is currently not supported.
    """
    searchContext: PartialMatchCriteria2 = None
    icChangesInfo: PartialMatchCriteria2 = None
    revisionChangesInfo: PartialMatchCriteria2 = None
    occEffChangesInfo: PartialMatchCriteria2 = None
    periodChangesInfo: PartialMatchCriteria2 = None


@dataclass
class StructureChangeDetailsResponseElement(TcBaseObj):
    """
    The StructureChangeDetailsResponseElement is returned from the getStructureChangeDetails method.   The main
    components of the structure are the following:
    
    - icChangeDetails - the details of a changed line from IC related change
    - revisionChangeDetails - the details of a changed line from revision related change     (Currently not supported) 
              
    - occEffChangeDetails - the details of a changed line from occurrence effectivity change (Currently not supported) 
                  
    - periodChangeDetails - the details of a changed line from period related change (Currently not supported)
    
    
                    
    
    :var icChangeDetails: The details of a changed line from IC related change 
    The PartialMatchCriteria2 is a structure of maps as follows:
    struct PartialMatchCriteria2
    { 
        StringToIntVectorMap2 intMap,
        StringToDblVectorMap2 dblMap,
        StringToStrVectorMap2 strMap,
        StringToObjVectorMap2 objMap,
        StringToDateVectorMap2 dateMap
    };
    
    Elements:
    intMap map of string to vector or integers. 
    dblMap map of string to vector of doubles. 
    strMap map of string to vector of strings. 
    objMap map of string to vector of objects. 
    dateMap map of string to vector of dates 
    Entries for the objMap:
    
    Key: ChangedLine (Required)    Value: The changed BOMLine
    
    Key: ICRev (Required)        Value: ItemRevision representing the Incremental Change object 
    
    Key: ICE (Required)        Value: IncrementalChangeElement associated with the changed line.
    
    Key: AffectedObject
    Value: An object of one of the following types (not a complete list):
    -         PSOccurrence
    -         Datasets
    -         Folders
    -         Forms
    -         Activities
    -         AbsOccData
    
    
    
    Entries for the strMap:
    
    Key: ChangeType (Required)
    Value: One of the following case sensitive strings:
        Add
        Remove
        Move
        Property
        Attachment
        Predecessor
     
    Key: HowConfigured    Value: the incremental change how configured string
           
    Key:AttributeName (Required for Property Change Type)
    Value: internal name of the attribute for attribute/property changes
           
    Key:AttributeValue (Required for Property Change Type)      
    Value: value of the attribute from incremental change        
          
    Key: PredecessorName (Required for Predecessor Change Type)
    Value: name of predecessor
    
    Key:PredecessorSequence (Required for Predecessor Change Type)
    Value: sequence number of predecessor     
     
    Key:UnitRangeText (Required for unit effectivity)
    Value: unit range text
    
    Key:DateRangeText (Required for date effectivity)
    Value: date range text
         
    :var revisionChangeDetails: This field is not currently supported.
    :var occEffChangeDetails: This field is not currently supported.
    :var periodChangeDetails: This field is not currently supported.
    """
    icChangeDetails: PartialMatchCriteria2 = None
    revisionChangeDetails: PartialMatchCriteria2 = None
    occEffChangeDetails: PartialMatchCriteria2 = None
    periodChangeDetails: PartialMatchCriteria2 = None


@dataclass
class TargetCreateOrUpdateParams(TcBaseObj):
    """
    Criteria for a target line that needs propagation status created or updated. In case a selected target line is to
    be moved from one parent to another, two such structures will be needed: one for the selected target line and the
    other for the suggested parent line. The structure for the selected target line will have a regular status value
    (e.g., "Handled") in its status field; and the structure for the suggested target line will have a special value
    "BDF3D5F9-1330-4ED0-A290-E6C1B5250DFF:MoveTo" in its status field.
    
    :var targetLine: The target line.
    :var incrementalChangeElement: Incremental change element (Optional)
    :var status: Propagation status
    """
    targetLine: BOMLine = None
    incrementalChangeElement: BusinessObject = None
    status: str = ''


@dataclass
class CreateOrUpdatePropagationDetailsIn(TcBaseObj):
    """
    The input criteria describing which target lines need propagation status updated or created.
    
    :var changedLine: The changed line.
    :var changedObject: The changed object, which can be IncrementalChangeElement,  ItemRevision, or PSOccurrence.
    :var changeType: The change type
    :var targets: The target info to create or update propagation statuses.    
    """
    changedLine: BOMLine = None
    changedObject: BusinessObject = None
    changeType: str = ''
    targets: List[TargetCreateOrUpdateParams] = ()


@dataclass
class CreateOrUpdatePropagationDetailsResp(TcBaseObj):
    """
    Propagation status response.
    
    :var details: List of propagation details elements.
    :var serviceData: Handle partial errors and return propagation statuses.
    """
    details: List[CreateOrUpdatePropagationDetailsRespElem] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdatePropagationDetailsRespElem(TcBaseObj):
    """
    List of propagation details.
    
    :var propagationDetails: List of propagation details.
    """
    propagationDetails: List[PropagationDetail] = ()


@dataclass
class CreateOrUpdateReviewStatusDetails(TcBaseObj):
    """
    Review status details set on a list of impacted target scopes.
    
    :var inputIndex: Corresponds to the index of the input list.
    :var statusDetails: Corresponds to the details of the review status associated with specific impacted target
    scopes. This list is in parallel with the impactedScopeStatus list in CreateOrUpdateReviewStatusIn.
    """
    inputIndex: int = 0
    statusDetails: List[CreateOrUpdateSingleReviewStatusDetail] = ()


@dataclass
class CreateOrUpdateReviewStatusIn(TcBaseObj):
    """
    Review status to be set on a list of impacted target scopes.
    
    :var changedLine: The changed BOMLine object.
    :var changedObject: The changed object, which can be IncrementalChangeElement, ItemRevision, or PSOccurrence.
    :var changeType: The type of the change.
    :var impactedScopeStatus: The review status for a list of impacted target scopes.        
    """
    changedLine: BOMLine = None
    changedObject: BusinessObject = None
    changeType: str = ''
    impactedScopeStatus: List[ReviewStatusInfo] = ()


@dataclass
class CreateOrUpdateReviewStatusResponse(TcBaseObj):
    """
    The review status details set on a list of impacted target scopes for every input.
    
    :var statusDetails: The review status details information. Each CreateOrUpdateReviewStatusDetails object in the
    statusDetails list represents the status details for a single CreateOrUpdateReviewStatusIn object in the input list.
    :var serviceData: Contains the client ID and partial errors.
    """
    statusDetails: List[CreateOrUpdateReviewStatusDetails] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateSingleReviewStatusDetail(TcBaseObj):
    """
    The review status set on a single impacted target scope.
    
    :var impactedScopeObject: The impacted scope object whose review status is set. It can be any of the following: the
    top most scope object which has a review status, the target scope itself if it has a review status, a child scope
    if this is the only child which has a review status, or null if the target scope has multiple children with a
    review status.
    :var reviewStatus: The review status from the statusScope.
    """
    impactedScopeObject: BusinessObject = None
    reviewStatus: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.Validation._2007_06.Validation import GetValResultsInput, GetValidationResultResponse, ValidationDataResponse, CreateOrUpdateValidationAgentsResponse, ValidationDataInfo, ValidationAgentInfo, ValidationResultsResponse, ValidationResultInfo
from typing import List
from tcsoa.base import TcService


class ValidationService(TcService):

    @classmethod
    def getValidationResults(cls, input: GetValResultsInput) -> GetValidationResultResponse:
        """
        This operation will return the found validation result objects based on the given validation target objects and
        the given validation checkers.
        
        Use cases:
        A validation result object is generated for a business object by a validation agent using the specialized
        validation logic that is defined by a validation data object. For example, the NX Check Mate validation agent
        will execute NX Check Mate tests to generate validation result objects for an NX part item revision object
        using the underlying UGMaster dataset object as a validation target.  This operation will be used to search for
        the validation result objects that a user will be interested in by reducing the search scope using the given
        input objects.
        """
        return cls.execute_soa_method(
            method_name='getValidationResults',
            library='Internal-Validation',
            service_date='2007_06',
            service_name='Validation',
            params={'input': input},
            response_cls=GetValidationResultResponse,
        )

    @classmethod
    def createOrUpdateValidationAgents(cls, inputs: List[ValidationAgentInfo]) -> CreateOrUpdateValidationAgentsResponse:
        """
        Creates or updates a group of validation agents.  When the ValidationAgentInfo.existingValidationAgent object
        is not NULL, this service will update the existing validation agent. If it is NULL, this service will create
        new validation agent. A validation agent of a given type will be uniquely identified by the object_name
        attribute of the validation agent item. If an existing validation agent of the given name, identified by
        ValidationAgentInfo.name structure member, does not exist, a new validation agent will be created. If an
        existing validation agent of the given name is found, a severe error will be returned while attempting to
        create a new validation agent.
        
        Use cases:
        This service will be used for a user to create new validation agents and/or update existing validation agents.
        A validation agent must be created before the user can utilize the validation application to execute validation
        business logic and create validation results.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateValidationAgents',
            library='Internal-Validation',
            service_date='2007_06',
            service_name='Validation',
            params={'inputs': inputs},
            response_cls=CreateOrUpdateValidationAgentsResponse,
        )

    @classmethod
    def createOrUpdateValidationData(cls, inputs: List[ValidationDataInfo]) -> ValidationDataResponse:
        """
        Creates or update a group of ValidationData objects.  A ValidationData object represents a validation checker
        of a certain validation agent in the database. When the ValidationDataInfo.existingValidationData object is not
        NULL, this service will update the existing ValidationData object. If it is NULL, this service will create new
        ValidationData object. A ValidationData object will be uniquely identified by the combination of
        validation_name attribute and validation_ application attribute of the ValidationData object. A new
        ValidationData object will be created if an existing ValidationData, identified by ValidationDataInfo.name and
        ValidationDataInfo.agentName structure members, does not exist. If an existing ValidationData object of the
        given name and agent name is found, a severe error will be returned while attempting to create a new
        ValidationData object.
        
        Use cases:
        This service will be used for a user to create new validation checkers and/or update existing validation
        checkers. A validation checker must be created before the user can utilize the validation application to
        execute validation business logic specified by the ValidationData object and create validation results.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateValidationData',
            library='Internal-Validation',
            service_date='2007_06',
            service_name='Validation',
            params={'inputs': inputs},
            response_cls=ValidationDataResponse,
        )

    @classmethod
    def createOrUpdateValidationResults(cls, inputs: List[ValidationResultInfo]) -> ValidationResultsResponse:
        """
        Creates or update a group of validation results.  When the ValidationResultInfo .existingValidationResult
        object is not NULL, this service will update the existing ValidationResult object. If
        ValidationResultInfo.existingValidationResult is NULL, this service will create new a new ValidationResult
        object. Before creating a new one, the service first attempts to validate the existence of the ValidationResult
        by the combination of ValidationResultInfo.targetObject, ValidationResultInfo.validationDataObject, and
        ValidationResultInfo.resultId structure members. If the ValidationResult object does not exist, a new
        ValidationResult object will be created. If a ValidationResult object already exists, an error message will be
        returned while attempting to create a new ValidationResult object.
        
        Use cases:
        The processes of a customer site may require that a part will be released to production only after the part
        meets compliance specified by business logic that is embedded with various validation checkers for certain
        validation application utility. After executing validation business logic on the part, the user will use this
        service to create new ValidationResult objects or update existing ValidationResult objects to demonstrate the
        compliance status of the part.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateValidationResults',
            library='Internal-Validation',
            service_date='2007_06',
            service_name='Validation',
            params={'inputs': inputs},
            response_cls=ValidationResultsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject, ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AssignPartnerContractInput(TcBaseObj):
    """
    selected WorkspaceObject,partner contract to assign, preferred status.
    
    :var partnerContract: Vm0PrtnrContractRevision to assign.
    :var preferredStatus: Preferred status for partner contract in the context of an object. Valid values are given in
    LOV preference, "Vm0PrtnrContractAssignmentStatus".
    :var selectedObject: Selected WorkspaceObject to assign partner contract.
    """
    partnerContract: ItemRevision = None
    preferredStatus: str = ''
    selectedObject: WorkspaceObject = None


@dataclass
class ViewModelObject(TcBaseObj):
    """
    Structure to store the Teamcenter BusinessObject and its associated view model properties to be displayed as a
    single row in the table.
    
    :var modelObject: Represents the underlying business object of the row. The object can be WorkspaceObject.
    :var viewModelProperties: A list of ViewModelProperty structures.
    """
    modelObject: BusinessObject = None
    viewModelProperties: List[ViewModelProperty] = ()


@dataclass
class ViewModelProperty(TcBaseObj):
    """
    Structure to store the properties of logical business object to be displayed as a single cell in the row of table.
    Properties which do not exist directly on the business object, only those properties will be created.
    
    :var propInternalName: Real name of the property.
    :var propDisplayName: Display name of the property.
    :var propDBValue: Database value of the property.
    :var propUIValue: Value of the property which will be shown to user.
    :var propDataType: Data type of the property. Supported values are: "string", "int", "boolean", "short", "float",
    "double".
    :var isEditable: If property is editable then "true", otherwise "false".
    :var hasLOV: If true, property has LOV attached; otherwise, false.
    :var propParentBO: The business object which has this property.
    """
    propInternalName: str = ''
    propDisplayName: str = ''
    propDBValue: str = ''
    propUIValue: str = ''
    propDataType: str = ''
    isEditable: bool = False
    hasLOV: bool = False
    propParentBO: BusinessObject = None


@dataclass
class ViewModelRowsResponse(TcBaseObj):
    """
    The structure holds information of the view model objects. Each ViewModelObject structure represents a row
    displayed on table.
    
    :var viewModelRows: List of view model rows for each displayed node in a table.
    :var serviceData: The service data object.
    """
    viewModelRows: List[ViewModelObject] = ()
    serviceData: ServiceData = None


@dataclass
class ColumnInfo(TcBaseObj):
    """
    1.1.1.4    ColumnInfo
    
    :var propInternalName: Internal name of the column.
    :var propDisplayName: Display name of the column.
    :var objectTypeName: The business object type (ItemRevision and its subtypes) of the property associated with
    column.
    :var columnWidth: Width of the column in pixels.
    :var isDisplayed: If column needs to be displayed or not.
    """
    propInternalName: str = ''
    propDisplayName: str = ''
    objectTypeName: str = ''
    columnWidth: int = 0
    isDisplayed: bool = False


@dataclass
class ColumnInfoInput(TcBaseObj):
    """
    Type of table and WorkspaceObject object.
    
    :var tableType: Type of table to render. Supported table type is "PartnersStructure".
    :var inputObject: WorkspaceObject business object.
    """
    tableType: str = ''
    inputObject: WorkspaceObject = None


@dataclass
class ColumnInfoResponse(TcBaseObj):
    """
    This operation returns the information of the columns for a table which is rendered as a part of tree table for
    given type of structure and business object.
    
    The column information returned as part of this operation includes display name, internal name, object type,  width
    and a flag to indicate if column needs to be displayed or not.
    
    :var columnInfo: A list containing the columns for a table with other attributes of the column.
    :var serviceData: Service data object associated with the operation
    """
    columnInfo: List[ColumnInfo] = ()
    serviceData: ServiceData = None


@dataclass
class RemovePartnerContractInput(TcBaseObj):
    """
    The structure contains -
    1. WorkspaceObject from which the Partner Contract is to be removed. This could be an ItemRevision or a structure
    context.
    2. Partner Contract which is to be removed from the input WorkspaceObject.
    
    :var objectForContractRemoval: The WorkspaceObject from which the Partner Contract is to be removed.
    :var partnerContract: The Vm0PrtnrContractRevision object which is to be removed.
    """
    objectForContractRemoval: WorkspaceObject = None
    partnerContract: ItemRevision = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Manufacturing._2016_03.ImportExport import NamesAndValuesMap, MfgExportToBriefcaseResponse, ImportManufaturingFeaturesInput
from tcsoa.gen.Manufacturing._2015_10.ImportExport import ImportManufaturingFeaturesResponse
from typing import List
from tcsoa.base import TcService


class ImportExportService(TcService):

    @classmethod
    def importManufacturingFeatures(cls, input: ImportManufaturingFeaturesInput) -> ImportManufaturingFeaturesResponse:
        """
        The PLMXML contains the data about the MFGs. The data contains the name of the MFGs. Their transform, their
        parts' connection, and their form attributes. There are two types of manufacturing features. Continuous MFG
        like arcweld ( Mfg0BVRArcWeld) and discrete MFGs like weld point (Mfg0BVRWeldPoint) and datum point
        (Mfg0BVRDatumPoint).
        For continuous MFGs, the PLMXML also contains the JT information.
        """
        return cls.execute_soa_method(
            method_name='importManufacturingFeatures',
            library='Manufacturing',
            service_date='2016_03',
            service_name='ImportExport',
            params={'input': input},
            response_cls=ImportManufaturingFeaturesResponse,
        )

    @classmethod
    def exportToBriefcase(cls, reason: str, sites: List[BusinessObject], objects: List[BusinessObject], transferOptionSet: BusinessObject, optionNameAndValues: NamesAndValuesMap, sessionOptionNamesAndValues: NamesAndValuesMap) -> MfgExportToBriefcaseResponse:
        """
        This operation is applicable specifically for Manufacturing Process Planner MPP  application.
        This operation combines following two operations.
        Teamcenter::Soa::GlobalMultiSite::_2008_06::ImportExport exportObjectsToOfflinePackage 
        Teamcenter::Soa::GlobalMultiSite::_2008_06::ImportExport requestExportToRemoteSites 
        In addition, it creates internal objects which are helpful in supplier collaboration use cases for
        manufacturing objects. Details in the use case section.
        
        Use cases:
        Use Case 1:  Exporting objects to the briefcase by transferring the ownership to the supplier.
        User wants to export Collaboration Context (CC) object in MPP to the briefcase to be used by the supplier at
        remote site. 
        The CC may contain product structure(s), bill of processes (BOP) such as plant BOP, plant structure etc.
        While exporting, the user wants to transfer ownership of few objects in the CC to the supplier so that the
        supplier can make changes to those objects on the other site.
        The user selects the CC and uses the menu option Tools, Export To, Briefcase...
        he menu option opens a dialog that allows user to set a destination site, a transfer option set, a list of
        traverse options and a list of session options. 
        In this case, all the objects which can be traversed by the transfer option set and session options will be
        exported into an output TC XML file. 
        The files and datasets related to exported objects will be downloaded and packed into the briefcase file along
        with the TC XML file.
        
        In addition, the internal object, Appearance Path Node (APN) will be created for the identified BOMLine objects
        in the CC. The BOMLine objects are identified based on the preference MERuleForBriefcaseExport.
        """
        return cls.execute_soa_method(
            method_name='exportToBriefcase',
            library='Manufacturing',
            service_date='2016_03',
            service_name='ImportExport',
            params={'reason': reason, 'sites': sites, 'objects': objects, 'transferOptionSet': transferOptionSet, 'optionNameAndValues': optionNameAndValues, 'sessionOptionNamesAndValues': sessionOptionNamesAndValues},
            response_cls=MfgExportToBriefcaseResponse,
        )



from __future__ import annotations

from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class LicAdminInput(TcBaseObj):
    """
    Key pair used to perform a licensing action.
    
    :var featureKey: The product as listed in the license file (e.g. teamcenter_author).
    :var licensingAction: The desired action. Valid values are:
    
    "init"                    Connect to the license server.
    "release"                Checks in the feature key back to the license server.
    "check"                Checks if the feature key exists in the license file.
    "exit"                    Checks in all open feature keys and disconnects from the license server.
    "get"                    Checks if the feature key exists in the license file  and then checks out the feature key.
    "init_get"            Connects to the license server (if not already connected, checks if the feature key exists in
    the license file  and then checks out the feature key.
    "release_exit"        Checks the feature key back to the license server and disconnects from the license server.
    """
    featureKey: str = ''
    licensingAction: str = ''
